/* $Workfile: RiPortIO.cpp $ */
   
/*
 * This console application is an example, demonstrating
 * the interface techniques to the RIEGL scanners.
 * Uncomment the respective sections in file riport.cpp
 * to see how to interface to
 *		RiPort.sys		WindowsNT Device Driver
 *		ECP mode		Windows95 Direct IO ECP mode using
 *						an ECP board.
 *		BYTE mode		Windows95 direct IO BYTE mode using
 *						a standard bidirectional printer port.
 *
 *    Company: RIEGL Laser Measurement Systems
 * Originator: Roland Schwarz
 *    $Author: rs $
 *      Start: 23.06.98
 *      $Date: 2003/06/11 06:19:36 $
 *   $Modtime: 15.07.98 9:23 $
 */

#include "stdio.h"
#include "windows.h"
#include "conio.h"
#include "riport.h"

// Prototypes
void Usage(void);
void OpenIfClosed(void);
void CloseIfOpen(void);
void doRead(int i);
void doReadHeader(int n);
void doLog(int n, char* szFilename);
void doByteLog(int n, char* szFilename);
void doDebug();

// Global data
typedef struct
{
	WORD wMeasPerLine;
	WORD wInternal0;
	BYTE bBytePerMeas;
	BYTE bNumFacets;
	WORD wLinePerImage;

} SubHeader1; // 3-D scanner

typedef struct
{
	WORD wMeasPerLine;
	WORD wInternal0;
	WORD wDataFormat; // refer to instrument handbook: chapter on ECP data  output
	WORD wLinePerImage;

} SubHeader2; // Ld90-3 types and line scanners

typedef struct
{
	DWORD dwSize;
	WORD wBytePerLine;
	BYTE bIdProtocol;

	BYTE bId;
	union
	{
		SubHeader1 sh1;
		SubHeader2 sh2;
	};
} Header;
Header m_Header = {sizeof(Header),0,0,0,0,0,0,0,0};

int		curUnit = 0;						// Current unit to test
BOOL	bIsOpen = FALSE;

// Main entry point
//
int main(int argc, char *argv[])
{
	int		val;
	char	ch;
	int		nArgs;
	int n;

	printf("RiPortIO Vers. 1.3 ");
	printf("in %s. \n", pszMode);

	char buf[256];
	char szFilename[_MAX_PATH];
	val = 1;
	do
	{
		printf(">");
		fgets(buf,255,stdin);
		nArgs = sscanf(buf,"%1s %d %s\n",&ch,&n,szFilename);
		if (1<nArgs) val = n;
		if (0<nArgs)
		{
			switch(ch)
			{
				case 'o':
				case 'O':
					OpenIfClosed();
					break;
				case 'u':
				case 'U':
					if (2==nArgs)
					{
						CloseIfOpen();
						curUnit = val;
						printf("Device %d selected\n", curUnit);
					}
					break;
				case 'c':
				case 'C':
					CloseIfOpen();
					break;
				case 'r':
				case 'R':
					if (1==nArgs)
					{
						if (m_Header.bIdProtocol==1)
							val = m_Header.wBytePerLine+2;
						else
							val = m_Header.wBytePerLine;
					}
					doRead(val);
					break;
				case 'h':
				case 'H':
					doReadHeader(0);
					break;
				case 'q':
				case 'Q':
					break;
				case 'l':
				case 'L':
					if (nArgs == 3)
					{
						doLog(val, szFilename);
					}
					break;
				case 'b':
				case 'B':
					if (nArgs == 3)
					{
						doByteLog(val, szFilename);
					}
					break;
				case 'd':
				case 'D':
					doDebug();
					break;
            default:
				printf("?\n");
				Usage();
			}
		}
		printf("\n");
	} while(ch!='q' && ch!='Q');

	CloseIfOpen();

	return 0;
}

// Open the device
//
void OpenIfClosed(void)
{

	if (!bIsOpen)
	{
		printf("Opening Device %d\n", curUnit);
		if (RiPortOpen(curUnit))
		{
			printf("Device found, sucessfully opened.\n");
			bIsOpen = TRUE;
		}
		else
			printf("ERROR: Could not open Device.\n");
	}
}


void CloseIfOpen(void)
{
	if (bIsOpen)
	{
		printf("Closing Device %d\n", curUnit);
		if (RiPortClose())
			printf("Device closed.\n");
		else
			printf("ERROR: Could not close Device.\n");
		
		m_Header.dwSize = 0;
		m_Header.wBytePerLine = 0;
		m_Header.bIdProtocol = 0;
		bIsOpen = FALSE;
	}
}


// Read 'n' bytes of data from the device
//
void doRead(int n)
{
	char	*buf;
	ULONG	nRead;
	ULONG u;

	buf = (char *) malloc(n);
	if (buf == NULL)
	{
		printf("Failed to allocate buffer for read");
		exit(1);
	}

	OpenIfClosed();

	// Read data from driver
	printf("Reading from device - ");

	RiPortRead(buf, n, &nRead);

	printf("%d bytes read from device (%d requested).\n", nRead, n);
	
	// Display received data
	printf("\n\t");
	for (u=0; u<nRead; u++)
	{
		printf("%02X ",(UCHAR)(buf[u]));
		if (15==u%16) printf("\n\t");
	}
	printf("\n\n");
	
	free(buf);
}

void Usage(void)
{
	printf("Usage:\n");
	printf("       u sets unit number to value specified (default = 0)\n");
	printf("       o opens the currently selected device\n");
	printf("       h reads the header record\n");
	printf("       r [n] initiates a read of specified number of bytes\n");
	printf("       l n file   logs n frames to specified file\n");
	printf("       b n file   logs n bytes to specified file\n");
	printf("       c close currently selcted device\n");
	printf("       q ends the program\n");
	printf("\n");
}

// Read the Header from the device
//
void doReadHeader(int n)
{

	ULONG	nRead;

	CloseIfOpen();
	OpenIfClosed();

	// Read data from driver
	printf("Reading from device - ");
	RiPortRead(&m_Header.dwSize, 4, &nRead);
	if (4==nRead)
	{
		printf("\nThe header is:\n");
		printf("   dwSize        = %d\n",m_Header.dwSize);

		RiPortRead(&m_Header.wBytePerLine, m_Header.dwSize-4, &nRead);
		if (m_Header.dwSize-4==nRead)
		{
			printf("   wBytePerLine  = %d\n",m_Header.wBytePerLine);
			printf("   bIdProtocol   = %d\n",m_Header.bIdProtocol);
			printf("   bId           = %d\n",m_Header.bId);
			if (m_Header.bId==1)
			{ 
				printf("   wMeasPerLine  = %d\n",m_Header.sh1.wMeasPerLine);
				printf("   wInternal0    = %d\n",m_Header.sh1.wInternal0);
				printf("   bBytePerMeas  = %d\n",m_Header.sh1.bBytePerMeas);
				printf("   bNumFacets    = %d\n",m_Header.sh1.bNumFacets);
				printf("   wLinePerImage = %d\n",m_Header.sh1.wLinePerImage);
				printf("\n");
			}
			else if (m_Header.bId==2)
			{ 
				printf("   wMeasPerLine  = %d\n",m_Header.sh2.wMeasPerLine);
				printf("   wInternal0    = %d\n",m_Header.sh2.wInternal0);
				printf("   wDataFormat   = %d\n",m_Header.sh2.wDataFormat);
				printf("   wLinePerImage = %d\n",m_Header.sh2.wLinePerImage);
				printf("\n");
			}
			else
			{
				printf("ERROR: unknown header identifier %d\n", m_Header.bId);
			}
			for (int i=0; i<n; i++)
				doRead(m_Header.wBytePerLine+2);			
		}
		else
			printf("%d bytes read from device (%d requested).\n", nRead, m_Header.dwSize-4);
	}
	else
		printf("%d bytes read from device (%d requested).\n", nRead, 4);
}

#define BUFFER_SIZE 1024

void doByteLog(int n, char* szFilename)
{
FILE* out;	
char	*buf;
ULONG	nRead;
int nToRead;
int nLeft;

    nLeft = n;
    buf = (char *) malloc(BUFFER_SIZE);
	if (buf == NULL)
	{
		printf("Failed to allocate buffer for read");
		exit(1);
	}

	if (NULL!=(out = fopen(szFilename, "wb")))
	{
        printf("logging to file %s, hit any key to abort.\n", szFilename);
        nToRead = (nLeft>BUFFER_SIZE)?BUFFER_SIZE:nLeft;
        while (nLeft) {
			RiPortRead(buf, nToRead, &nRead);
            nLeft -= nRead;
            if (nRead) {
                fwrite(buf, nRead,1,out);
                printf("Left: %8d\r");
            }
            nToRead = (nLeft>BUFFER_SIZE)?BUFFER_SIZE:nLeft;
            if (kbhit()) break;
        }
        printf("\n");
		if (kbhit()) getch();
	
		fclose(out);
	}

    free(buf);
}

void doLog(int n, char* szFilename)
{
FILE* out;	
char	*buf;
ULONG	nRead;
int nToRead;	
	if (NULL!=(out = fopen(szFilename, "wb")))
	{
		doReadHeader(0);

		if (m_Header.bIdProtocol==1)
			nToRead = m_Header.wBytePerLine+2;
		else
			nToRead = m_Header.wBytePerLine;

		buf = (char *) malloc(nToRead);
		if (buf == NULL)
		{
			printf("Failed to allocate buffer for read");
			exit(1);
		}
		printf("logging to file %s, hit any key to abort.\n", szFilename);
		fwrite(&m_Header, m_Header.dwSize, 1, out);
		for (int i=0; i<n ;i++)
		{
			printf("Logging Frame %d of %d ...\n", i+1, n);
			for (int j=0; j<m_Header.sh1.wLinePerImage; j++)
			{
				RiPortRead(buf, nToRead, &nRead);
				if(nToRead!=(WORD)nRead)
				{
					printf("%d bytes read from device (%d requested).\n", nRead, nToRead);
				}
				else
				{
					fwrite(buf, nToRead, 1, out);
				}
				if (kbhit()) break;
			}
			if (kbhit()) break;
		}
		if (kbhit()) getch();
	
		fclose(out);
	}
}

void doDebug()
{
char	*lbuf;
short int nSync;
ULONG nRead;

	CloseIfOpen();
	OpenIfClosed();

	RiPortRead(&m_Header.dwSize, 4, &nRead);
	if (4==nRead)
	{
		printf("\nThe header is:\n");
		printf("   dwSize        = %d\n",m_Header.dwSize);

		RiPortRead(&m_Header.wBytePerLine, 2, &nRead);
		if (2==nRead)
		{
			printf("   wBytePerLine  = %d\n",m_Header.wBytePerLine);
		}
	}

	lbuf = (char *) malloc(m_Header.wBytePerLine+2);
	if (lbuf == NULL)
	{
		printf("Failed to allocate buffer for read");
		exit(1);
	}

	RiPortRead(lbuf, m_Header.dwSize-6, &nRead);

	printf ("Starting\n");

	do {
		RiPortRead(&nSync, 2, &nRead);
		if (2 != nRead) {
			printf("Timeout\n");
			break;
		}
		if (nSync != m_Header.wBytePerLine) {
			printf("%4x : %4x\n", nSync, m_Header.wBytePerLine);
			break;
		}
		RiPortRead(lbuf, m_Header.wBytePerLine, &nRead);
		if (m_Header.wBytePerLine != nRead) {
			printf("Timeout\n");
			break;
		}
	}
	while (!kbhit());
	if (kbhit()) getch();

	free(lbuf);
}
