// CtrlDlg.cpp : implementation file
//

#include "stdafx.h"
#include "riscan2d.h"
#include "CtrlDlg.h"

#include <atlbase.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WM_SCANCNF (WM_APP+100)
class CNativeException {};

/////////////////////////////////////////////////////////////////////////////
// CCtrlDlg dialog


CCtrlDlg::CCtrlDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCtrlDlg::IDD, pParent)
{
	m_pScanCnf = NULL;
	m_fIsOpen = FALSE;
	//{{AFX_DATA_INIT(CCtrlDlg)
	m_dlLineStart = 0.0;
	m_dlLineInc = 0.0;
	m_dlLaserRate = 0.0;
	m_dlLineStop = 0.0;
	m_strLineFPS = _T("0.00");
	//}}AFX_DATA_INIT
}


void CCtrlDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCtrlDlg)
	DDX_Text(pDX, IDC_LINE_START, m_dlLineStart);
	DDX_Text(pDX, IDC_LINE_INC, m_dlLineInc);
	DDX_Text(pDX, IDC_LASER_RATE, m_dlLaserRate);
	DDX_Text(pDX, IDC_LINE_STOP, m_dlLineStop);
	DDX_Text(pDX, IDC_LINE_FPS, m_strLineFPS);
	//}}AFX_DATA_MAP
}

const int CCtrlDlg::BusyMap[] = {
	IDC_LINE_START,
	IDC_LINE_INC,
	IDC_LINE_NUMBER,
	IDC_LASER_RATE,
	IDC_APPLY,
	IDC_CALC,
	IDC_PARK,
	0
};

BEGIN_MESSAGE_MAP(CCtrlDlg, CDialog)
	//{{AFX_MSG_MAP(CCtrlDlg)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	ON_BN_CLICKED(IDC_CALC, OnCalc)
	ON_BN_CLICKED(IDC_CLOSE, OnClose)
	ON_BN_CLICKED(IDC_PARK, OnPark)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_SCANCNF, OnScannerNotify)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCtrlDlg message handlers

LRESULT CCtrlDlg::OnScannerNotify(WPARAM wParam, LPARAM lParam)
{
	if (1 == LOWORD(wParam)) { // chek Id that has been specified in the open call

		switch(HIWORD(wParam)) { // dispatch notifications
		case SCFN_OPEN:  OnScannerOpen();        break;
		case SCFN_CLOSE: OnScannerClose();       break;
		case SCFN_APPLY: OnScannerApply();       break;
		case SCFN_CALC:  OnScannerCalc();        break;
		case SCFN_PARK:  OnScannerPark();        break;
		case SCFN_ERROR: OnScannerError(lParam); break;
		default:
			AfxMessageBox(IDS_ERROR_UNEXPECTED);
		}

	}
	else
		AfxMessageBox(IDS_ERROR_UNEXPECTED);

	return 0;
}

int CCtrlDlg::ErrorBox(UINT nIDPrompt, UINT nType)
{
CComBSTR bstr;
CString strError;
int nError;

	m_pScanCnf->GetErrNmb(&nError);
	m_pScanCnf->GetErrStr(nError, &bstr);
	strError.Format(nIDPrompt, (LPCSTR)CString((BSTR)bstr));

	return AfxMessageBox(strError, nType);
}

void CCtrlDlg::OnScannerError(int nError)
{
	switch(nError) {
	case SCF_ERROR_BAD_APPLY: // it did not work, ask the user if he wants a suggestion
		if (IDYES == AfxMessageBox(IDS_BADAPPLY_CALC, MB_YESNO)) {
			m_pScanCnf->Calc();
		}
		break;
	case SCF_ERROR_CANNOT_OPEN: // give the user a chance to retry
		if (IDRETRY == ErrorBox(IDS_ERROR_SCANNER , MB_RETRYCANCEL|MB_ICONERROR)) {
			m_pScanCnf->Open((_bstr_t)m_strPort,(DWORD)m_hWnd, WM_SCANCNF, 1);
			Busy();
		}
		break;
	default:
		ErrorBox(IDS_ERROR_SCANNER, MB_OK|MB_ICONERROR);
	}
}

void CCtrlDlg::OnScannerOpen()
{
CString str;

	// retrieve the scanner type and display in caption
	GetWindowText(str);
	str += " - ";
	str += m_pScanCnf->bstrType;
	SetWindowText(str);

	// get the current scanner settings and fill dialog box
	m_dlLineStart  = m_pScanCnf->dTheta;
	m_dlLineInc    = m_pScanCnf->dDeltaTheta;
	m_dlLineStop  = m_dlLineStart+(m_pScanCnf->nlNumTheta-1)*m_dlLineInc;
	m_dlLaserRate  = m_pScanCnf->dClock;
	m_strLineFPS.Format("%.2f", 1.0/m_pScanCnf->dTotalTime);

	// Q560 access of native interface
	if (0 == strcmp(m_pScanCnf->bstrType, "Q560") ){
		// The following lines demonstrate how the scanner may be accessed by native commands.
		// Please note that you might disturb the RiSCANLIB's correct function,
		// by issuing commands that interfere with the libraries code.
		try {
			// Choose the measurement mode.
			NativePutInt(1, "SCN_OPMODE");
			NativePutInt(1, "LOG_MODE");
			// The subsampling factor will be corrected
			// during the Calc function of the library
			// if necessary.
			NativePutInt(4, "SCN_MSM");
			// This example simply closes any open files and
			// disables logging. You might want to setup your
			// recorder instead. Make sure you choose the correct
			// triggering mode too.
			NativeCommand("HDLOG_CLOSE");
		}
		catch (CNativeException) {
			ErrorBox(IDS_ERROR_SCANNER, MB_OK|MB_ICONERROR);
		}
	}
	
	UpdateData(FALSE);
	Busy(FALSE);
	m_fIsOpen = TRUE;
}

void CCtrlDlg::OnScannerClose()
{
	m_fIsOpen = FALSE;
	EndDialog(IDOK);
}

void CCtrlDlg::OnScannerApply()
{
	m_strLineFPS.Format("%.2f", 1.0/m_pScanCnf->dTotalTime);
	UpdateData(FALSE);
	Busy(FALSE); // activate user input fields
}

void CCtrlDlg::OnScannerCalc()
{
	// get the calculated scanner settings and fill dialog box
	m_dlLineStart  = m_pScanCnf->dTheta;
	m_dlLineInc    = m_pScanCnf->dDeltaTheta;
	m_dlLineStop  = m_dlLineStart+(m_pScanCnf->nlNumTheta-1)*m_dlLineInc;
	m_dlLaserRate  = m_pScanCnf->dClock;
	m_strLineFPS.Format("%.2f", 1.0/m_pScanCnf->dTotalTime);
	UpdateData(FALSE);
	Busy(FALSE); // activate user input fields
}

void CCtrlDlg::OnScannerPark() {
	Busy(FALSE);
}


void CCtrlDlg::Busy(BOOL bBusy)
{
CWnd* pWnd;
	for (int n=0; 0 != BusyMap[n]; ++n) {
		if (0 != (pWnd = GetDlgItem(BusyMap[n])))
			pWnd->EnableWindow(!bBusy);
	}
}

BOOL CCtrlDlg::OnInitDialog() 
{
	m_strLineFPS.Format("%.2f", 0.0);

	CDialog::OnInitDialog();
	m_strPort = AfxGetApp()->GetProfileString(SECTION, _T("CmdPort"), _T(""));	

	m_pScanCnf.CreateInstance(__uuidof(ScanCnf));
	m_fIsOpen = FALSE;
	if (m_pScanCnf) {
		m_pScanCnf->Open((_bstr_t)m_strPort,(DWORD)m_hWnd, WM_SCANCNF, 1);
		Busy();
	}
	else { // there was a fatal error, perhaps the DLL has not been installed correctly
		AfxMessageBox(IDS_ERROR_INIT, MB_ICONEXCLAMATION);
		EndDialog(IDCANCEL);
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CCtrlDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	if (m_pScanCnf) m_pScanCnf.Release();
}

void CCtrlDlg::OnApply() 
{
	if (UpdateData()) {
		m_pScanCnf->dTheta      = m_dlLineStart;
		m_pScanCnf->dDeltaTheta = m_dlLineInc;
		if (0 != m_dlLineInc)
			m_pScanCnf->nlNumTheta  = (int)((m_dlLineStop-m_dlLineStart)/m_dlLineInc) +1;
		else
			m_pScanCnf->nlNumTheta = 32768;
		m_pScanCnf->dClock      = m_dlLaserRate;
		m_pScanCnf->Apply();
		Busy(); // deactivate user input fields
	}
}

void CCtrlDlg::OnCalc() 
{
	if (UpdateData()) {
		m_pScanCnf->dTheta      = m_dlLineStart;
		m_pScanCnf->dDeltaTheta = m_dlLineInc;
		if (0 != m_dlLineInc)
			m_pScanCnf->nlNumTheta  = (int)((m_dlLineStop-m_dlLineStart)/m_dlLineInc) +1;
		else
			m_pScanCnf->nlNumTheta = 32768;
		m_pScanCnf->dClock      = m_dlLaserRate;
		m_pScanCnf->Calc();
		Busy(); // deactivate user input fields
	}
}

void CCtrlDlg::OnPark() 
{
	IScanCtrl2Ptr pScanCtrl;
	pScanCtrl = m_pScanCnf;

	if (UpdateData() && NULL != pScanCtrl) {
		pScanCtrl->Park();
		Busy();
	}
}

void CCtrlDlg::OnClose() 
{
	if (UpdateData()) {
		if (m_fIsOpen) {
			m_pScanCnf->Close();
			Busy();
		}
		else
			EndDialog(IDOK);
	}
}

void CCtrlDlg::NativePutInt(int nVal, _bstr_t bstrName)
{
	INativeCnfPtr pNative = m_pScanCnf;
    if (S_OK != pNative->PutInt(nVal, bstrName, 1000))
        throw CNativeException();
}

void CCtrlDlg::NativeCommand(_bstr_t bstrName)
{
	INativeCnfPtr pNative = m_pScanCnf;
    if (S_OK != pNative->Command(bstrName, 1000))
        throw CNativeException();
}
