// RiSCAN2DDoc.cpp : implementation of the CRiSCAN2DDoc class
//

#include "stdafx.h"
#include "RiSCAN2D.h"

#include "RiSCAN2DDoc.h"
#include "RiSCAN2DView.h"
#include "PortSettings.h"
#include "CtrlDlg.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DDoc

IMPLEMENT_DYNCREATE(CRiSCAN2DDoc, CDocument)

BEGIN_MESSAGE_MAP(CRiSCAN2DDoc, CDocument)
	//{{AFX_MSG_MAP(CRiSCAN2DDoc)
	ON_COMMAND(ID_SET_PORTS, OnSetPorts)
	ON_UPDATE_COMMAND_UI(ID_SET_PORTS, OnUpdateSetPorts)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_COMMAND(ID_SET_SCANNER, OnSetScanner)
	ON_UPDATE_COMMAND_UI(ID_SET_SCANNER, OnUpdateSetScanner)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_START_SCANNING, OnStartScanning)
    ON_UPDATE_COMMAND_UI(ID_START_SCANNING, OnUpdateStartScanning)
	ON_COMMAND(ID_STOP_SCANNING, OnStopScanning)
    ON_UPDATE_COMMAND_UI(ID_STOP_SCANNING, OnUpdateStopScanning)
  	ON_COMMAND(ID_START_LOGGING, OnStartLogging)
	ON_UPDATE_COMMAND_UI(ID_START_LOGGING, OnUpdateStartLogging)
  	ON_COMMAND(ID_STOP_LOGGING, OnStopLogging)
	ON_UPDATE_COMMAND_UI(ID_STOP_LOGGING, OnUpdateStopLogging)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DDoc construction/destruction

CRiSCAN2DDoc::CRiSCAN2DDoc() : m_pScanner(__uuidof(Scanner))
{
    m_dZoom = 10.0f;
    m_dRotate = 0.0f;
    m_nCurrentGrid = 4;
    m_dGrid.Add(0.0f);
    m_dGrid.Add(0.1f);
    m_dGrid.Add(0.5f);
    m_dGrid.Add(1.0f);
    m_dGrid.Add(5.0f);
    m_dGrid.Add(10.0f);
    m_dGrid.Add(50.0f);
    m_dGrid.Add(100.0f);
    m_eState = Stop;
    m_nXGridSize = 0;
    m_nYGridSize = 0;
    AfxGetApp()->WriteProfileInt(SECTION, _T("Logging"), 0);
    m_bIsLogging = 0;
    AfxGetApp()->WriteProfileInt(SECTION, _T("Envelope"), 0);
}

CRiSCAN2DDoc::~CRiSCAN2DDoc()
{
}

BOOL CRiSCAN2DDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

    m_bFileOpen = FALSE;
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DDoc serialization

void CRiSCAN2DDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DDoc diagnostics

#ifdef _DEBUG
void CRiSCAN2DDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CRiSCAN2DDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DDoc commands
/*************************************************************************
 *
 * void OnSetPorts()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function will open a dialog box to set the ports
 *
 *************************************************************************/
void CRiSCAN2DDoc::OnSetPorts() 
{
	CPortSettings Dlg;
    Dlg.DoModal();
}

void CRiSCAN2DDoc::OnSetScanner() 
{
	CCtrlDlg dlg;
	dlg.DoModal();
}


/*************************************************************************
 *
 * BOOL OnOpenDocument(LPCTSTR lpszPathName)
 *
 * Parameters:
 *
 * lpszPathName         - Points to the path of the document to be opened
 *
 * Return value: 
 *
 * BOOL                 - Nonzero if the document was successfully loaded;
 *                        otherwise 0
 *
 * Description:
 *
 * Will be called when a document should be opened.
 *
 *************************************************************************/
BOOL CRiSCAN2DDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	// Get View pointer
    POSITION pos = GetFirstViewPosition();
	CView* pFirstView = GetNextView( pos );
    m_bstrFileName = lpszPathName;
    m_bFileOpen = TRUE;
	
	return TRUE;
}


/*************************************************************************
 *
 * void OnStartScanning()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function get the ports and start the scan.
 *
 *************************************************************************/
void CRiSCAN2DDoc::OnStartScanning() 
{
    // Get View pointer
    POSITION pos = GetFirstViewPosition();
    CRiSCAN2DView* pFirstView = (CRiSCAN2DView*)GetNextView(pos);
    pFirstView->m_nLinesReceived = 0;
    pFirstView->m_nLines = 0;
    pFirstView->m_nMeas = 0;

    _bstr_t bstrDataPort;
	_bstr_t bstrCmdPort;


	if (!m_bFileOpen) {
        bstrDataPort = AfxGetApp()->GetProfileString(SECTION, _T("DataPort"), _T(""));// _T("\\\\.\\RiPT0"));
		bstrCmdPort = AfxGetApp()->GetProfileString(SECTION, _T("CmdPort"), _T(""));
	}
    else {
        bstrDataPort = m_bstrFileName.copy();
		bstrCmdPort = "";
	}

    int i = AfxGetApp()->GetProfileInt(SECTION, _T("Logging"), 0);

    if (i)
    {
        CFileDialog FileDlg (FALSE, NULL, _T("*.2dd"),
                             OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                             _T("2dd File (*.2dd)|*.2dd|All Files (*.*)|*.*|"), AfxGetApp()->m_pMainWnd);
        if (IDOK == FileDlg.DoModal())
        {
            CString strFileName = FileDlg.GetPathName();

            m_pScanner->OpenEx(bstrDataPort, bstrCmdPort, _bstr_t(strFileName), (DWORD)pFirstView->m_hWnd,
                             SCN_MLINE|SCN_MFRAME, WM_SCANNER, 0);

            //m_pScanner->Open(bstrDataPort, _bstr_t(strFileName), (DWORD)pFirstView->m_hWnd,
            //                 SCN_MLINE|SCN_MFRAME, WM_SCANNER);
            m_eState = Start;
        }
    }
    else
    {
        m_pScanner->OpenEx(bstrDataPort, bstrCmdPort, _T(""), (DWORD)pFirstView->m_hWnd,
                         SCN_MLINE|SCN_MFRAME, WM_SCANNER, 0);

        //m_pScanner->Open(bstrDataPort, _T(""), (DWORD)pFirstView->m_hWnd,
        //                 SCN_MLINE|SCN_MFRAME, WM_SCANNER);
        m_eState = Start;
    }
}


/*************************************************************************
 *
 * void OnStopScanning()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function will stop the current scan.
 *
 *************************************************************************/
void CRiSCAN2DDoc::OnStopScanning() 
{
    m_eState = Stop;
    m_pScanner->Close();
}


/*************************************************************************
 *
 * void OnUpdateStartScanning(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the scanning start button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateStartScanning(CCmdUI* pCmdUI) 
{
    if (m_eState == Start)
    {
        pCmdUI->Enable(FALSE);
        CMainFrame* pFrm = (CMainFrame*)AfxGetApp()->m_pMainWnd;
        pFrm->m_wndLog.EnableWindow(FALSE);
    }
    else
        pCmdUI->Enable(TRUE);
}


/*************************************************************************
 *
 * void OnUpdateStopScanning(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the scanning stop button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateStopScanning(CCmdUI* pCmdUI) 
{
    if (m_eState == Stop)
    {
        pCmdUI->Enable(FALSE);
        CMainFrame* pFrm = (CMainFrame*)AfxGetApp()->m_pMainWnd;
        pFrm->m_wndLog.EnableWindow(TRUE);
    }
    else
        pCmdUI->Enable(TRUE);
}


/*************************************************************************
 *
 * void OnUpdateSetPorts(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the set ports button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateSetPorts(CCmdUI* pCmdUI) 
{
	if ((!m_bFileOpen) && (m_eState == Stop))
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}

void CRiSCAN2DDoc::OnUpdateSetScanner(CCmdUI* pCmdUI) 
{
	if ((!m_bFileOpen) && (m_eState == Stop))
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}


/*************************************************************************
 *
 * void OnUpdateFileOpen(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the open file button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	if (m_eState == Stop)
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}


/*************************************************************************
 *
 * void OnUpdateFileNew(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the new file button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	if (m_eState == Stop)
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}


/*************************************************************************
 *
 * void OnStartLogging()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function starts logging.
 *
 *************************************************************************/
void CRiSCAN2DDoc::OnStartLogging() 
{
	m_pScanner->Log(TRUE);
    m_bIsLogging = TRUE;
}


/*************************************************************************
 *
 * void OnStartLogging()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function stops logging.
 *
 *************************************************************************/
void CRiSCAN2DDoc::OnStopLogging() 
{
	m_pScanner->Log(FALSE);
    m_bIsLogging = FALSE;
}



/*************************************************************************
 *
 * void OnUpdateStartLogging(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the start logging button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateStartLogging(CCmdUI* pCmdUI) 
{
    int i = AfxGetApp()->GetProfileInt(SECTION, _T("Logging"), 0);
    if ((i) && (!m_bIsLogging))
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}


/*************************************************************************
 *
 * void OnUpdateStopLogging(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the stop logging button.
 * 
 *************************************************************************/
void CRiSCAN2DDoc::OnUpdateStopLogging(CCmdUI* pCmdUI) 
{
    int i = AfxGetApp()->GetProfileInt(SECTION, _T("Logging"), 0);
    if ((i) && (m_bIsLogging))
        pCmdUI->Enable(TRUE);
    else
        pCmdUI->Enable(FALSE);
}


