// RiSCAN2DView.cpp : implementation of the CRiSCAN2DView class
//

#include "stdafx.h"
#include "RiSCAN2D.h"

#include "RiSCAN2DDoc.h"
#include "RiSCAN2DView.h"
#include <math.h>

#include "scandsys.h"

#ifndef __GNUC__
float RDFP(void* pv, int ofs)
{
	float sd;
	*(DWORD*)(&sd) = RD32(pv,ofs);
	return sd;
}

void WRFP(void* pv, float d, int ofs)
{
	DWORD dw;
	*(float*)(&dw)=d;
	WR32(pv,dw,ofs);
}
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView

IMPLEMENT_DYNCREATE(CRiSCAN2DView, CView)

BEGIN_MESSAGE_MAP(CRiSCAN2DView, CView)
	//{{AFX_MSG_MAP(CRiSCAN2DView)
	ON_WM_LBUTTONDOWN()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONUP()
	ON_WM_TIMER()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
	// Standard printing commands
  	//ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	//ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	//ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
    ON_MESSAGE(WM_SCANNER, OnScannerNotify)
    ON_UPDATE_COMMAND_UI(ID_NEXT, OnUpdateNext)
    ON_UPDATE_COMMAND_UI(ID_RUN, OnUpdateRun)
    ON_COMMAND(ID_ZOOM_PLUS, OnZoomPlus)
    ON_COMMAND(ID_ZOOM_PLUS_10, OnZoomPlus10)
    ON_COMMAND(ID_ZOOM_MINUS, OnZoomMinus)
    ON_COMMAND(ID_ZOOM_MINUS_10, OnZoomMinus10)
    ON_COMMAND(ID_ROTATE_LEFT, OnRotateLeft)
    ON_COMMAND(ID_ROTATE_RIGHT, OnRotateRight)
    ON_COMMAND(ID_GRID_PLUS, OnGridPlus)
    ON_COMMAND(ID_GRID_MINUS, OnGridMinus)
    ON_COMMAND(ID_NEXT, OnNext)
    ON_COMMAND(ID_RUN, OnRun)
    ON_COMMAND(ID_OPTIMIZE, OnOptimize)
    ON_UPDATE_COMMAND_UI(ID_OPTIMIZE, OnUpdateOptimize)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView construction/destruction

CRiSCAN2DView::CRiSCAN2DView()
{
    m_pdVertex = NULL;

// Q500 SPECIFIC START
	m_pHeader = NULL;
	m_pLineRaw = NULL;
	m_pLine = NULL;
	m_nLineSize = 0;
	m_dTiltAngleZ = 80;
	m_dTiltAngleY = 100.0;
// Q500 SPECIFIC END

}

CRiSCAN2DView::~CRiSCAN2DView()
{
    free(m_pdVertex);

// Q500 SPECIFIC START
	free(m_pHeader);
	free(m_pLineRaw);
// Q500 SPECIFIC END
}

BOOL CRiSCAN2DView::PreCreateWindow(CREATESTRUCT& cs)
{
	// Set cross cursor for window
	cs.lpszClass = AfxRegisterWndClass(
    CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW, // use any window styles
    AfxGetApp()->LoadStandardCursor(IDC_SIZEALL),
    (HBRUSH) (COLOR_WINDOW + 1));         // background brush
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView drawing

void CRiSCAN2DView::OnDraw(CDC* pDC)
{
    int i, nX, nY;
    CPen Pen;

	CRiSCAN2DDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
    CPoint ptZero = m_pDoc->m_ptZeroPoint;
    pDC->SetBkMode(TRANSPARENT);

    BITMAP bmpInfo;
	(m_pDoc->m_bmp).GetBitmap(&bmpInfo);

    int n = AfxGetApp()->GetProfileInt(SECTION, _T("Envelope"), 0);
    if ((!n) || (m_bEnvelope))
    {
        // Paint Grid
        pDC->SetStretchBltMode(COLORONCOLOR);
        pDC->StretchBlt(0, 0, bmpInfo.bmWidth, bmpInfo.bmHeight, &(m_pDoc->m_dc), 
                        0, 0, bmpInfo.bmWidth, bmpInfo.bmHeight, SRCCOPY);
        m_bEnvelope = FALSE;
    }

    // Paint Pixels
    for (i=0; i<m_nMeas; i++)
    {
        float dR = m_pdVertex[3*i];
        float dT = m_pdVertex[3*i+1];
        dT += m_pDoc->m_dRotate;
        nX = (int)floor(ptZero.x + (m_pDoc->m_dZoom * (dR * cos(dT))));
        nY = (int)floor(ptZero.y - (m_pDoc->m_dZoom * (dR * sin(dT))));
        pDC->SetPixelV(nX, nY, 0);//RGB(255,255,255));
    }

	// display the scanner specific data
// Q500 SPECIFIC START
	if (m_pLineRaw) {
		CString str;
		str.Format("Tilt Angle Y: % 12.3f", m_dTiltAngleY);
		pDC->TextOut(10, 10, str);
		str.Format("Tilt Angle Z: % 12.3f", m_dTiltAngleZ);
		pDC->TextOut(10, 30, str);
	}
// Q500 SPECIFIC END

}

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView printing

BOOL CRiSCAN2DView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CRiSCAN2DView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CRiSCAN2DView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView diagnostics

#ifdef _DEBUG
void CRiSCAN2DView::AssertValid() const
{
	CView::AssertValid();
}

void CRiSCAN2DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CRiSCAN2DDoc* CRiSCAN2DView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRiSCAN2DDoc)));
	return (CRiSCAN2DDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRiSCAN2DView message handlers
void CRiSCAN2DView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

    m_pDoc = GetDocument();

    CRect Rect;
    GetClientRect(&Rect);
    m_pDoc->m_ptZeroPoint = CPoint(Rect.right/2,Rect.bottom/2);

    m_nLines = 0;
    m_nMeas = 0;
    m_nLinesReceived = 0;
    m_pDoc->m_dZoom = 50.0f;
    m_pDoc->m_dRotate = 0.0f;
    m_pDoc->m_nCurrentGrid = 2;
    m_ptMouseDown = CPoint(0,0);
    m_bMove = FALSE;
    m_bEnvelope = TRUE;
    m_pDoc->m_eState = Stop;

    // get the current window's resolution
    CDC* pddc = GetDC();
    m_pDoc->m_nXGridSize = pddc->GetDeviceCaps(HORZRES);
    m_pDoc->m_nYGridSize = pddc->GetDeviceCaps(VERTRES);

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CReflectorView::OnScannerNotify message handlers

/*************************************************************************
 *
 * LRESULT OnScannerNotify(WPARAM wParam, LPARAM lParam)
 *
 * Parameters:
 *
 * WPARAM wParam        - HIWORD(MESSAGE), LOWORD(wId)
 * LPARAM lParam        - Contains message specific information such as
 *                        current line number, frame number or error number
 *
 * Return value: 
 *
 * LRESULT              - Returns zero
 *
 * Description:
 *
 * Is called when the scanner sends a message.
 * 
 *************************************************************************/
LRESULT CRiSCAN2DView::OnScannerNotify(WPARAM wParam, LPARAM lParam)
{
    // Handle scanner messages
	switch (wParam) {
		case SCN_OPEN:  OnScannerOpen   (wParam, lParam);   break;
		case SCN_LINE:  OnScannerLine   (wParam, lParam);   break;
        case SCN_FRAME: OnScannerFrame  (wParam, lParam);   break; 
		case SCN_CLOSE: OnScannerClose  (wParam, lParam);   break;
		case SCN_EOF:   OnScannerEOF    (wParam, lParam);   break;
		case SCN_ERROR: OnScannerError  (wParam, lParam);   break;
        case SCN_LOGSTARTED: OnScannerStartLog(wParam, lParam); break;
        case SCN_LOGSTOPPED: OnScannerStopLog(wParam, lParam); break;
	}

	return 0;
}// --- End OnScannerNotify ---


/*************************************************************************
 *
 * void OnScannerOpen(WPARAM wParam, LPARAM lParam)
 *
 * Parameters:
 *
 * WPARAM wParam        - HIWORD(MESSAGE), LOWORD(wId)
 * LPARAM lParam        - Contains message specific information such as
 *                        current line number, frame number or error number
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when the scanner was succesfully opened.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerOpen(WPARAM wParam, LPARAM lParam)
{
long fValid;
int nHeaderSize;
int nLines;
int nSize;

    m_pDoc->m_pScanner->Log(FALSE);
    m_pDoc->m_pScanner->GetDimension(&(m_nLines), &(m_nMeas));

    int n;
    // prepare memory
    m_pdVertex = (float*)realloc(m_pdVertex, 3*m_nMeas*sizeof(float));
	for (n=0; n<3*m_nMeas; n++) *(m_pdVertex+n) = 0.0;

// Q500 SPECIFIC START
	// Handling of scanner specific data. i.e Q500 tilt angles:
	// get access to the raw header bytes
	m_pDoc->m_pScanner->GetRawInfo(&fValid, &nHeaderSize, &m_nLineSize, &nLines);
	m_pHeader = realloc(m_pHeader, nHeaderSize);
	m_pDoc->m_pScanner->GetRawHeader(&nSize, (unsigned char*)m_pHeader, nHeaderSize);

	// now make sure, that the expected information is contained in the data
	if (10 == Header_bID_RD(m_pHeader) && 
		4 == Header10_bParameterId_RD(m_pHeader) &&
		6 == Header10_bTrailerId_RD(m_pHeader) &&
		3 <= Header10_wTrailerIdSub_RD(m_pHeader) ) { // we need at minimum the 6.3 Trailer
		// ok, the data is there, prepare a buffer space
		m_pLineRaw = realloc(m_pLineRaw, m_nLineSize); // this inludes the sync bytes
		m_pLine = (unsigned char*)m_pLineRaw + ((1&Header_bProtocolId_RD(m_pHeader))?2:0); // consider sync offset
	}
// Q500 SPECIFIC END

  	m_pDoc->m_pScanner->NextFrame();
}// --- End OnScannerOpen ---


/*************************************************************************
 *
 * void OnScannerLine(WPARAM wParam, LPARAM lParam)
 *
 * Parameters:
 *
 * WPARAM wParam        - HIWORD(MESSAGE), LOWORD(wId)
 * LPARAM lParam        - Contains message specific information such as
 *                        current line number, frame number or error number
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when the scanner was received a line.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerLine(WPARAM wParam, LPARAM lParam)
{
    int nLine;
    int nTransfered;

    nLine = lParam;

   	(m_pDoc->m_pScanner)->GetVertex(&nTransfered, m_pdVertex, SCN_POLAR_MATH|SCN_LOCAL,
                                        3, nLine*m_nMeas, m_nMeas);

// Q500 SPECIFIC START
	// read in the tilt angles
	if (m_pLineRaw) {
		int nSize;
		m_pDoc->m_pScanner->GetRawData(&nSize, 0, (unsigned char*)m_pLineRaw, m_nLineSize);
		m_dTiltAngleY = (360.0/400.0)*Parameter4_0_dAngleUnit_RD(m_pHeader)*Trailer6_3_dwTiltAngleY_RD(m_pHeader, m_pLine);
		m_dTiltAngleZ = (360.0/400.0)*Parameter4_0_dAngleUnit_RD(m_pHeader)*Trailer6_3_dwTiltAngleZ_RD(m_pHeader, m_pLine);
	}
// Q500 SPECIFIC END

    m_nLinesReceived++;

    Invalidate();
}// --- End OnScannerLine ---


/*************************************************************************
 *
 * void OnScannerFrame(WPARAM wParam, LPARAM lParam)
 *
 * Parameters:
 *
 * WPARAM wParam        - HIWORD(MESSAGE), LOWORD(wId)
 * LPARAM lParam        - Contains message specific information such as
 *                        current line number, frame number or error number
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when the scanner was received a frame.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerFrame(WPARAM wParam, LPARAM lParam)
{
    if (!m_pDoc->m_bFileOpen)
    {
        m_pDoc->m_pScanner->NextFrame();
    }
    Invalidate();
}// --- End OnScannerFrame ---


/*************************************************************************
 *
 * void OnScannerClose()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when the scanner was succesfully closed.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerClose(WPARAM wParam, LPARAM lParam)
{
	KillTimer(1);

// Q500 SPECIFIC START
    free(m_pHeader);
	free(m_pLineRaw);
	m_pHeader = NULL;
	m_pLineRaw = NULL;
	m_pLine = NULL;
// Q500 SPECIFIC END

    //AfxMessageBox("Scanner Close");
}// --- End OnScannerClose ---


/*************************************************************************
 *
 * void OnScannerEOF()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when the scanner sends a EndOfFile message.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerEOF(WPARAM wParam, LPARAM lParam)
{
    m_pDoc->m_eState = Stop;
    m_pDoc->m_pScanner->Close();
    KillTimer(1);
    MessageBeep(MB_ICONHAND);
    MessageBox(_T("End of file"), _T("Error"), MB_OK|MB_ICONEXCLAMATION);
}// --- End OnScannerEOF ---


/*************************************************************************
 *
 * void OnScannerError()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * Is called when an error occurs.
 *
 *************************************************************************/
void CRiSCAN2DView::OnScannerError(WPARAM wParam, LPARAM lParam)
{
	CString strError;
	BSTR BSTRError;
	int nError;

    m_pDoc->m_eState = Stop;
    KillTimer(1);
    (m_pDoc->m_pScanner)->GetErrNmb(&nError);
	(m_pDoc->m_pScanner)->GetErrStr(nError, &BSTRError);

	_bstr_t bstrError(BSTRError,FALSE);
	strError.Format(_T("Scanner Error: %s"), (LPTSTR)bstrError);
    MessageBeep(MB_ICONHAND);
	MessageBox(strError, _T("Error"), MB_OK|MB_ICONEXCLAMATION);

	(m_pDoc->m_pScanner)->Close();
}// --- End OnScannerError ---


void CRiSCAN2DView::OnScannerStartLog(WPARAM wParam, LPARAM lParam)
{
}


void CRiSCAN2DView::OnScannerStopLog(WPARAM wParam, LPARAM lParam)
{
}


/*************************************************************************
 *
 * void OnLButtonDown(UINT nFlags, CPoint point) 
 *
 * Parameters:
 *
 * UINT nFlags          - Indicates whether various virtual keys are down.
 * CPoint point         - Specifies the x- and y-coordinate of the cursor.
 *
 * Return value: 
 *
 * Description:
 *
 * The framework calls this member function when the user presses the left mouse button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnLButtonDown(UINT nFlags, CPoint point) 
{
    if (nFlags == (MK_LBUTTON|MK_CONTROL))
    {
        m_pDoc->m_ptZeroPoint = point;
        SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
        PaintGrid();
        m_bEnvelope = TRUE;
        Invalidate();
    }
    else
    {
        m_ptMouseDown = point;
        m_bMove = TRUE;
    }

	CView::OnLButtonDown(nFlags, point);
}


/*************************************************************************
 *
 * void OnLButtonDown(UINT nFlags, CPoint point) 
 *
 * Parameters:
 *
 * UINT nFlags          - Indicates whether various virtual keys are down.
 * CPoint point         - Specifies the x- and y-coordinate of the cursor.
 *
 * Return value: 
 *
 * Description:
 *
 * The framework calls this member function when the user releases the left mouse button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnLButtonUp(UINT nFlags, CPoint point) 
{
    if (m_bMove)
    {
        m_pDoc->m_ptZeroPoint.x += (point.x - m_ptMouseDown.x);
        m_pDoc->m_ptZeroPoint.y += (point.y - m_ptMouseDown.y);

        m_bMove = FALSE;
        SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
        PaintGrid();
        m_bEnvelope = TRUE;
        Invalidate();
    }
    
	CView::OnLButtonUp(nFlags, point);
}


/*************************************************************************
 *
 * void OnEraseBkgnd(CDC* pDC)
 *
 * Parameters:
 *
 * CDC* pDC         - Specifies the device-context object
 *
 * Return value: 
 *
 * BOOL             - Nonzero if it erases the background; otherwise 0
 *
 * Description:
 *
 * The framework calls this member function when the CWnd object background
 * needs erasing (for example, when resized). It is called to prepare an
 * invalidated region for painting.
 *
 *************************************************************************/
BOOL CRiSCAN2DView::OnEraseBkgnd(CDC* pDC) 
{
    return FALSE;
    //return CView::OnEraseBkgnd(pDC);
}


/*************************************************************************
 *
 * void OnUpdateNext(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the status bar.
 * 
 *************************************************************************/
void CRiSCAN2DView::OnUpdateNext(CCmdUI* pCmdUI) 
{
    if ((m_pDoc->m_eState == Stop) || (!m_pDoc->m_bFileOpen))
        pCmdUI->Enable(FALSE);
    else
        pCmdUI->Enable(TRUE);
}



/*************************************************************************
 *
 * void OnUpdateRun(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the status bar.
 * 
 *************************************************************************/
void CRiSCAN2DView::OnUpdateRun(CCmdUI* pCmdUI) 
{
    if ((m_pDoc->m_eState == Stop) || (!m_pDoc->m_bFileOpen))
        pCmdUI->Enable(FALSE);
    else
        pCmdUI->Enable(TRUE);
}


/*************************************************************************
 *
 * void OnPaint()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * The framework calls this member function when Windows or an application
 * makes a request to repaint a portion of an applications window. 
 * Clears the area which is right and under the bitmap.
 *
 *************************************************************************/
void CRiSCAN2DView::OnPaint() 
{

    CPaintDC dc(this); // device context for painting
    OnPrepareDC(&dc);
	if (dc.m_ps.fErase) {
		BITMAP bmpInfo;
 		(m_pDoc->m_bmp).GetBitmap(&bmpInfo);

        CRect rc((dc.m_ps).rcPaint);
    	dc.DPtoLP(rc);
	    
 		int w,h;
		w = rc.right - bmpInfo.bmWidth;
        h = rc.bottom - bmpInfo.bmHeight;
 
 		if (w > 0) 
 			dc.PatBlt(bmpInfo.bmWidth, rc.top, w, rc.Height(), WHITENESS);
 			dc.PatBlt(rc.left, bmpInfo.bmHeight, rc.Width(), h, WHITENESS);
	}
	OnDraw(&dc);
}// --- End OnPaint ---


/*************************************************************************
 *
 * void SetScanSize(int nWidth, int nHeight)
 *
 * Parameters:
 *
 * int nWidth       - specified the width of the bitmap
 * int nHeight      - specified the height of the bitmap
 *
 * Return value: 
 *
 * Description:
 *
 * Creates a compatible bitmap and dc.
 *
 *************************************************************************/
void CRiSCAN2DView::SetGridSize(int nWidth, int nHeight)
{
	m_pDoc->m_bmp.DeleteObject();
	m_pDoc->m_dc.DeleteDC();

    CClientDC dc(this);

    m_pDoc->m_bmp.CreateCompatibleBitmap(&dc, nWidth, nHeight);
	m_pDoc->m_dc.CreateCompatibleDC(&dc);
	m_pDoc->m_dc.SelectObject(&m_pDoc->m_bmp);

	EraseGrid();
}// --- End SetScanSize ---


/*************************************************************************
 *
 * void EraseScan()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * Erase bitmap and dc.
 *
 *************************************************************************/
void CRiSCAN2DView::EraseGrid()
{
	BITMAP bmpInfo;
	m_pDoc->m_bmp.GetBitmap(&bmpInfo);
	m_pDoc->m_dc.PatBlt(0, 0, bmpInfo.bmWidth, bmpInfo.bmHeight, PATCOPY);
}// --- End EraseScan ---


/*************************************************************************
 *
 * void PaintGrid()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function is painting the grid.
 *
 *************************************************************************/
void CRiSCAN2DView::PaintGrid()
{
    CPen Pen;
    CPoint ptZero = m_pDoc->m_ptZeroPoint;
    if (m_pDoc->m_nCurrentGrid > 0)
    {
        int i;
        Pen.CreatePen(PS_DOT, 1, RGB(0,0,255));
        m_pDoc->m_dc.SelectObject(Pen);

        m_pDoc->m_dc.MoveTo(0, ptZero.y);
        m_pDoc->m_dc.LineTo(m_pDoc->m_nXGridSize, ptZero.y);
        i = int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        while ((ptZero.y - i) >= 0)
        {
            m_pDoc->m_dc.MoveTo(0, ptZero.y - i);
            m_pDoc->m_dc.LineTo(m_pDoc->m_nXGridSize, ptZero.y - i);
            i += int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        }
        i = int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        while (i<=m_pDoc->m_nYGridSize)
        {
            m_pDoc->m_dc.MoveTo(0, ptZero.y + i);
            m_pDoc->m_dc.LineTo(m_pDoc->m_nXGridSize, ptZero.y + i);
            i += int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        }

        m_pDoc->m_dc.MoveTo(ptZero.x, 0);
        m_pDoc->m_dc.LineTo(ptZero.x, m_pDoc->m_nYGridSize);
        i = int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        while ((ptZero.x - i) >=0)
        {
            m_pDoc->m_dc.MoveTo(ptZero.x - i, 0);
            m_pDoc->m_dc.LineTo(ptZero.x - i, m_pDoc->m_nYGridSize);
            i += int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        }
        i = int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        while (i<=m_pDoc->m_nXGridSize)
        {
            m_pDoc->m_dc.MoveTo(ptZero.x + i, 0);
            m_pDoc->m_dc.LineTo(ptZero.x + i, m_pDoc->m_nYGridSize);
            i += int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid]);
        }
        Pen.DeleteObject();
        // Paint Zero Point
    }
    Pen.CreatePen(PS_SOLID, 3, RGB(255,0,0));
    m_pDoc->m_dc.SelectObject(Pen);
    m_pDoc->m_dc.MoveTo(ptZero.x - 6, ptZero.y);
    m_pDoc->m_dc.LineTo(ptZero.x + 7, ptZero.y);
    m_pDoc->m_dc.MoveTo(ptZero.x, ptZero.y - 6);
    m_pDoc->m_dc.LineTo(ptZero.x, ptZero.y + 7);
    Pen.DeleteObject();
}


/*************************************************************************
 *
 * void OnZoomPlus()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the zoom "+1" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnZoomPlus() 
{
    if (m_pDoc->m_dZoom > 1.0f)
        m_pDoc->m_dZoom++;
    else
        m_pDoc->m_dZoom *= 2.0f;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;

    Invalidate();
}


/*************************************************************************
 *
 * void OnZoomPlus10()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the zoom "+10" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnZoomPlus10() 
{
    if (m_pDoc->m_dZoom >= 1.0f)
        m_pDoc->m_dZoom += 10.0f;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;

    Invalidate();
}


/*************************************************************************
 *
 * void OnZoomMinus()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the zoom "-1" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnZoomMinus() 
{
    if (m_pDoc->m_dZoom > 1.0f)
        m_pDoc->m_dZoom--;
    else
        m_pDoc->m_dZoom /= 2.0f;

    if (int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid])<5)
        m_pDoc->m_nCurrentGrid = 0;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;

    Invalidate();
}


/*************************************************************************
 *
 * void OnZoomMinus10()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the zoom "-10" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnZoomMinus10() 
{
    if (m_pDoc->m_dZoom > 1.0f)
    {
        m_pDoc->m_dZoom -= 10.0f;
        if (m_pDoc->m_dZoom < 1)
            m_pDoc->m_dZoom = 1;
    }

    if (int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid])<5)
        m_pDoc->m_nCurrentGrid = 0;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;

    Invalidate();
}


/*************************************************************************
 *
 * void OnRotateLeft()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the rotate "Left" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnRotateLeft() 
{
    m_pDoc->m_dRotate += float(10.0 * (PI/180.0));
    if (m_pDoc->m_dRotate > (2*PI - float(10.0 * (PI/180.0))))
        m_pDoc->m_dRotate = 0.0f;
    Invalidate();
}


/*************************************************************************
 *
 * void OnRotateRight()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the rotate "Right" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnRotateRight() 
{
    m_pDoc->m_dRotate -= float(10.0 * (PI/180.0));
    if (m_pDoc->m_dRotate < (2*PI*(-1) + float(10.0 * (PI/180.0))))
        m_pDoc->m_dRotate = 0.0f;
    Invalidate();
}


/*************************************************************************
 *
 * void OnGridPlus()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the "Grid+" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnGridPlus() 
{
    if (m_pDoc->m_nCurrentGrid > 6)
        m_pDoc->m_nCurrentGrid = 7;
    else
        m_pDoc->m_nCurrentGrid++;

    while ((int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid])<5)
            && (m_pDoc->m_nCurrentGrid < 7))
    {
        m_pDoc->m_nCurrentGrid++;
    }
    if (int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid])<5)
        m_pDoc->m_nCurrentGrid = 0;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;
    Invalidate();
}


/*************************************************************************
 *
 * void OnGridMinus()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the "Grid-" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnGridMinus() 
{
    if (m_pDoc->m_nCurrentGrid < 1)
        m_pDoc->m_nCurrentGrid = 0;
    else
        m_pDoc->m_nCurrentGrid--;

    if (int(m_pDoc->m_dZoom * m_pDoc->m_dGrid[m_pDoc->m_nCurrentGrid])<5)
        m_pDoc->m_nCurrentGrid = 0;

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;

    Invalidate();
}


/*************************************************************************
 *
 * void OnNext()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the "Next" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnNext() 
{
    m_pDoc->m_pScanner->NextFrame();
    Invalidate();
}


/*************************************************************************
 *
 * void OnRun()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * This function handles a press on the "Run" button.
 *
 *************************************************************************/
void CRiSCAN2DView::OnRun() 
{
    SetTimer(1, 20, 0);
}


/*************************************************************************
 *
 * void OnTimer(UINT nIDEvent)
 *
 * Parameters:
 *
 * UINT nIDEvent        - Specifies the identifier of the timer.
 *
 * Return value: 
 *
 * Description:
 *
 * The framework calls this member function after each interval specified
 * in the SetTimer member function used to install a timer.
 *************************************************************************/
void CRiSCAN2DView::OnTimer(UINT nIDEvent) 
{
    if (nIDEvent == 1)
    {
        if (m_pDoc->m_eState == Stop)
            KillTimer(1);
        else if (S_OK == m_pDoc->m_pScanner->NextFrame())
        {
            //KillTimer(1);
        }
        else
            KillTimer(1);

        Invalidate();
    }

	CView::OnTimer(nIDEvent);
}


/*************************************************************************
 *
 * void OnOptimize()
 *
 * Parameters:
 *
 * Return value: 
 *
 * Description:
 *
 * Calculates the best settings and apply them.
 *************************************************************************/
void CRiSCAN2DView::OnOptimize()
{
    int i;
    float dMax = 0.0f;
    for (i=0; i<m_nMeas; i++)
    {
        if (m_pdVertex[3*i] > dMax)
            dMax = m_pdVertex[3*i];
    }
    CRect Rect;
    GetClientRect(&Rect);
    float dClient = Rect.Height() / m_pDoc->m_dZoom;
    while ((dMax > dClient) && (m_pDoc->m_dZoom > 1))
    {
        m_pDoc->m_dZoom -= 1.0f;
        dClient = Rect.Height() / m_pDoc->m_dZoom;
    }
    m_pDoc->m_nCurrentGrid = 0;
    m_pDoc->m_ptZeroPoint = CPoint(Rect.Width()/2, Rect.Height());

    SetGridSize(m_pDoc->m_nXGridSize, m_pDoc->m_nYGridSize);
    PaintGrid();
    m_bEnvelope = TRUE;
    Invalidate();
}


/*************************************************************************
 *
 * void OnUpdateOptimize(CCmdUI* pCmdUI)
 *
 * Parameters:
 *
 * CCmdUI* pCmdUI       - This object represents the menu item or toolbar
 *                        button or other user-interface object that
 *                        generated the command
 *
 * Return value: 
 *
 * Description:
 *
 * This function updates the optimize button.
 * 
 *************************************************************************/
void CRiSCAN2DView::OnUpdateOptimize(CCmdUI* pCmdUI)
{
    if (m_pDoc->m_eState == Stop)
    {
        pCmdUI->Enable(FALSE);
    }
    else
        pCmdUI->Enable(TRUE);
}
