/*
  Program:     Scanner Interface Library
  Version:     2.2
  Module:      $Workfile: ScanDsys.h $
  Language:    C++, C
  Start:       23.08.99
  Date:        $Date: 2003/07/09 17:00:53 $
  Modtime:     $Modtime: 16.10.01 11:00 $
  Fileversion: $Revision: 1.1 $
  Originator:  Roland Schwarz
  Author:      $Author: rs $

Copyright (c) 1999 Riegl Laser Measurement Systems.

The authors hereby grant permission to use, and copy this software for the
purpose of enhancing the useability of Riegl Laser Measurement Systems GmbH
(RLMS) instruments you own. You may NOT distribute or modify the software 
for the use in commercial applications without the written consent of RLMS.
In any case copyright notices and this notice must be retained in all copies
verbatim in any distributions.

IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE IS PROVIDED ON AN
"AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef __SCANDSYS_H__
#define __SCANDSYS_H__

#ifdef __cplusplus
extern "C" {
#endif

/* PORTING TO FOREIGN SYSTEMS:
 * at least you must make sure you have the following macros defined
 * BYTE  the compiler  8-bit data type usually char
 * WORD  the compiler 16-bit data type
 * DWORD the compiler 32-bit data type 
 */
 
#ifndef _WINDEF_
#  ifdef __GNUC__
#    include <sys/types.h>
#    ifndef _BITS_TYPES_H
#	   define DWORDINT int
#      define DWORD unsigned int
#      define WORD unsigned short int
#      define BYTE unsigned char
#    else
#	   define DWORDINT __int32_t
#      define DWORD __uint32_t
#      define WORD  __uint16_t
#      define BYTE  __uint8_t
#  endif
#  define MAXDWORD    0xffffffff
#  define MAXWORD     0xffff
#  define MAXBYTE     0xff
#  ifndef __cplusplus
#    define bool BYTE
#    define true (1)
#    define false (0)
#  endif
#else
#  define DWORDINT __int32
#  define DWORD unsigned __int32
#  define MAXDWORD    0xffffffff
#  define WORD  unsigned __int16
#  define MAXWORD     0xffff
#  define BYTE  unsigned __int8
#  define MAXBYTE     0xff
#  ifndef __cplusplus
#    define bool BYTE
#    define true (1)
#    define false (0)
#  endif
#endif

#  ifndef NOMINMAX
#    ifndef max
#      define max(a,b)            (((a) > (b)) ? (a) : (b))
#    endif
#    ifndef min
#      define min(a,b)            (((a) < (b)) ? (a) : (b))
#    endif
#  endif

/* this are the portability macros, shouldn't need to be touched */
#define LOBYTE(w)   ((BYTE) (w))
#define HIBYTE(w)   ((BYTE) (((WORD) (w) >> 8) & 0xFF))
#define LOWORD(l)   ((WORD) (l))
#define HIWORD(l)   ((WORD) (((DWORD) (l) >> 16) & 0xFFFF))
#define MAKEWORD(a, b) ((WORD) (((BYTE) (a)) | ((WORD) ((BYTE) (b))) << 8))

#endif

#define MAKEDWORD(a, b) ((DWORD) (((WORD) (a)) | ((DWORD) ((WORD) (b))) << 16))

#define WR8(pv, b, ofs)   { *((BYTE*)(pv)+(ofs))   = (BYTE)(b); }

#define WR16(pv, w, ofs)  { *((BYTE*)(pv)+(ofs))   = LOBYTE(w); *((BYTE*)(pv)+(ofs)+1) = HIBYTE(w); }
 
#define WR24(pv, dw, ofs) { *((BYTE*)(pv)+(ofs))   = LOBYTE(LOWORD(dw));\
                            *((BYTE*)(pv)+(ofs)+1) = HIBYTE(LOWORD(dw));\
							*((BYTE*)(pv)+(ofs)+2) = LOBYTE(HIWORD(dw)); }


#define WR32(pv, dw, ofs) { *((BYTE*)(pv)+(ofs))   = LOBYTE(LOWORD(dw));\
							*((BYTE*)(pv)+(ofs)+1) = HIBYTE(LOWORD(dw));\
							*((BYTE*)(pv)+(ofs)+2) = LOBYTE(HIWORD(dw));\
							*((BYTE*)(pv)+(ofs)+3) = HIBYTE(HIWORD(dw)); }


#ifdef __GNUC__
#define WRFP(pv, d, ofs) ({ DWORD dw; *(float*)(&dw)=d; WR32(pv,dw,ofs);})
#else
void WRFP(void* pv, float d, int ofs);
#endif

#define RD8(pv, ofs) ((BYTE)*((BYTE*)(pv)+(ofs)))

#define RD16(pv, ofs) MAKEWORD(*((BYTE*)(pv)+(ofs)), *((BYTE*)(pv)+(ofs)+1))

#define RD24(pv, ofs) MAKEDWORD(MAKEWORD(*((BYTE*)(pv)+(ofs)), *((BYTE*)(pv)+(ofs)+1)),\
					  MAKEWORD(*((BYTE*)(pv)+(ofs)+2), 0))

#define RD32(pv, ofs) MAKEDWORD(MAKEWORD(*((BYTE*)(pv)+(ofs)), *((BYTE*)(pv)+(ofs)+1)),\
					  MAKEWORD(*((BYTE*)(pv)+(ofs)+2), *((BYTE*)(pv)+(ofs)+3)))

#ifdef __GNUC__
#define RDFP(pv, ofs) ({ float sd; *(DWORD*)(&sd) = RD32(pv,ofs); sd; })
#else
float RDFP(void* pv, int ofs);
#endif

#define OFS(pv,i,l) ((BYTE*)(pv)+(i)*(l))

#define BITCAST8 BYTE
#define BITCAST16 WORD
#define BITCAST24 DWORD
#define BITCAST32 DWORD

/* base may be one of 8, 16, 24, 32 */
#define WRBITS(pv, f, ofs, base, s, l) WR##base((pv), (((f)&((1<<(l))-1))<<(s))\
	| (RD##base((pv), (ofs)) & ~(((1<<(l))-1)<<(s))), (ofs));

#define RDBITS(pv, ofs, base, s, l) (BITCAST##base)((RD##base(pv,ofs)>>s)&((1<<l)-1))

/*
 * Macros used to access the various fields of the header and
 * measurement records.
 * Common header information:
 */

#define Header_dwHeaderSize_WR(pv,dw) WR32(pv, dw, 0)
#define Header_dwHeaderSize_RD(pv)    RD32(pv,     0)
#define Header_wDatasetLen_WR(pv,w)   WR16(pv,  w, 4)
#define Header_wDatasetLen_RD(pv)     RD16(pv,     4)
#define Header_bProtocolId_WR(pv,b)    WR8(pv,  b, 6)
#define Header_bProtocolId_RD(pv)      RD8(pv,     6)
#define Header_bID_WR(pv,b)            WR8(pv,  b, 7)
#define Header_bID_RD(pv)              RD8(pv,     7)

/*
 * Macro to read/write the sync value
 */

#define Sync_RD(pv) RD16(pv, 0)
#define Sync_WR(pv, w) WR16(pv, w, 0)
#define Crc_RD(ph,pd)	RD16(pd, (Header_wDatasetLen_RD(ph)+((1&Header_bProtocolId_RD(ph))?2:0)))	
#define Crc_WR(ph,pd,w)	WR16(pd, w, (Header_wDatasetLen_RD(ph)+((1&Header_bProtocolId_RD(ph))?2:0)))	

/*
 * Macros used to access the various fields of the header and
 * measurement records.
 * ID1 specific:
 */

#define Header1_wMeasPerLine_WR(pv,w)   WR16(pv, w,  8)
#define Header1_wMeasPerLine_RD(pv)     RD16(pv,     8)
#define Header1_bBytePerMeas_WR(pv,b)    WR8(pv, b, 12)
#define Header1_bBytePerMeas_RD(pv)      RD8(pv,    12)
#define Header1_bNumFacets_WR(pv,b)      WR8(pv, b, 13)
#define Header1_bNumFacets_RD(pv)        RD8(pv,    13)
#define Header1_wLinesPerImage_WR(pv,w) WR16(pv, w, 14)
#define Header1_wLinesPerImage_RD(pv)   RD16(pv,    14)

#define OFSM1(ph,pd,i) ((BYTE*)(OFS(pd,i,Header1_bBytePerMeas_RD(ph))))
#define OFST1(ph,pd) ((BYTE*)(OFS(pd,Header1_wMeasPerLine_RD(ph),Header1_bBytePerMeas_RD(ph))))

#define Data1_wRange_WR(ph,pd,i,w)         WR16(OFSM1(ph,pd,i), w, 0)
#define Data1_wRange_RD(ph,pd,i)           RD16(OFSM1(ph,pd,i),    0)
#define Data1_wPolar_WR(ph,pd,i,w)         WR16(OFSM1(ph,pd,i), w, 2)
#define Data1_wPolar_RD(ph,pd,i)           RD16(OFSM1(ph,pd,i),    2)
#define Data1_wAzimuth_WR(ph,pd,i,w)       WR16(OFSM1(ph,pd,i), w, 4)
#define Data1_wAzimuth_RD(ph,pd,i)         RD16(OFSM1(ph,pd,i),    4)
#define Data1_fUpscan_WR(ph,pd,i,f)        WRBITS(OFSM1(ph,pd,i), f, 6, 16, 0, 1)
#define Data1_fUpscan_RD(ph,pd,i)          RDBITS(OFSM1(ph,pd,i),    6, 16, 0, 1)
#define Data1_wIntensity_WR(ph,pd,i,w)     WRBITS(OFSM1(ph,pd,i), w, 6, 16, 4,12)
#define Data1_wIntensity_RD(ph,pd,i)       RDBITS(OFSM1(ph,pd,i),    6, 16, 4,12)

#define LineControl1_wLineNumber_WR(ph,pd,w) WR16(OFST1(ph,pd), w, 0)
#define LineControl1_wLineNumber_RD(ph,pd)   RD16(OFST1(ph,pd),    0)
#define LineControl1_fUpscan_WR(ph,pd,f)     WRBITS(OFST1(ph,pd), f, 6, 16, 0, 1)
#define LineControl1_fUpscan_RD(ph,pd)       RDBITS(OFST1(ph,pd),    6, 16, 0, 1)
#define LineControl1_fFrame_WR(ph,pd,f)      WRBITS(OFST1(ph,pd), f, 6, 16, 3, 1)
#define LineControl1_fFrame_RD(ph,pd)        RDBITS(OFST1(ph,pd),    6, 16, 3, 1)

/*
 * Macros used to access the various fields of the header and
 * measurement records.
 * ID2 specific:
 */

#define Header2_wMeasPerLine_WR(pv,w)   WR16(pv, w,  8)
#define Header2_wMeasPerLine_RD(pv)     RD16(pv,     8)
#define Header2_wDataFormat_WR(pv,w)    WR16(pv, w, 12)
#define Header2_wDataFormat_RD(pv)      RD16(pv,    12)

/* the following field is missing from the original definition */
/* it is computed on the fly, (do not use in repetitive loops, performance...) */
#define Header2_bBytePerMeas_RD(pv) ( \
((Header2_wDataFormat_RD(pv) & 0x01)?2:0) + \
((Header2_wDataFormat_RD(pv) & 0x04)?1:0) + \
((Header2_wDataFormat_RD(pv) & 0x20)?1:0) )


#define OFSM2(ph,pd,i) ((BYTE*)(OFS(pd,i,Header1_bBytePerMeas_RD(ph))))

#define DYN_Data2_OFS(rd, ph, mask, def) (((Header2_wDataFormat_RD(ph))&(mask))? rd : def)

/* initialize the array of dynamic offsets */
/* only range amplitude and intensity can actually be present */
/* altough other bits might be set !! */
#define Data2_OFS(ph, ofs) { int n; n=0; \
ofs[0] = n; n += (Header2_wDataFormat_RD(ph) & 0x01)?2:0; \
ofs[1] = 0; \
ofs[2] = n; n += (Header2_wDataFormat_RD(ph) & 0x04)?1:0; \
ofs[3] = 0; \
ofs[4] = 0; \
ofs[5] = n; n += (Header2_wDataFormat_RD(ph) & 0x20)?1:0; \
ofs[6] = n; n += (Header2_wDataFormat_RD(ph) & 0x40)?3:0; \
}

#define Data2_wRange_WR(ph,pd,i,ofs,w)	                WR16(OFS(pd,i,ofs[6]), w, ofs[0])
#define Data2_wRange_RD(ph,pd,i,ofs)	  DYN_Data2_OFS(RD16(OFS(pd,i,ofs[6]),    ofs[0]), ph, 0x01, 0)
#define Data2_bIntensity_WR(ph,pd,i,ofs,b)              WR8( OFS(pd,i,ofs[6]), b, ofs[2])
#define Data2_bIntensity_RD(ph,pd,i,ofs)  DYN_Data2_OFS(RD8 (OFS(pd,i,ofs[6]),    ofs[2]), ph, 0x04, 0)
#define Data2_bQuality_WR(ph,pd,i,ofs,b)                WR8( OFS(pd,i,ofs[6]), b, ofs[5])
#define Data2_bQuality_RD(ph,pd,i,ofs)    DYN_Data2_OFS(RD8 (OFS(pd,i,ofs[6]),    ofs[5]), ph, 0x20, 0)

/*
 * Macros used to access the various fields of the header and
 * measurement records.
 * ID10 specific:
 */

#define Header10_wMeasOffset_WR(pv,w)       WR16(pv, w, 8)
#define Header10_wMeasOffset_RD(pv)         RD16(pv,    8)
#define Header10_wMeasSize_WR(pv,w)         WR16(pv, w,10)
#define Header10_wMeasSize_RD(pv)           RD16(pv,   10)
#define Header10_wMeasCount_WR(pv,w)        WR16(pv, w,12)
#define Header10_wMeasCount_RD(pv)          RD16(pv,   12)
#define Header10_bLeadinId_WR(pv,b)          WR8(pv, b,14)
#define Header10_bLeadinId_RD(pv)            RD8(pv,   14)
#define Header10_wLeadinIdSub_WR(pv,w)      WR16(pv, w,15)
#define Header10_wLeadinIdSub_RD(pv)        RD16(pv,   15)
#define Header10_bMeasId_WR(pv,b)            WR8(pv, b,17)
#define Header10_bMeasId_RD(pv)              RD8(pv,   17)
#define Header10_wMeasIdSub_WR(pv,w)        WR16(pv, w,18)
#define Header10_wMeasIdSub_RD(pv)          RD16(pv,   18)
#define Header10_bTrailerId_WR(pv,b)         WR8(pv, b,20)
#define Header10_bTrailerId_RD(pv)           RD8(pv,   20)
#define Header10_wTrailerIdSub_WR(pv,w)     WR16(pv, w,21)
#define Header10_wTrailerIdSub_RD(pv)       RD16(pv,   21)
#define Header10_bParameterId_WR(pv,b)       WR8(pv, b,23)
#define Header10_bParameterId_RD(pv)         RD8(pv,   23)
#define Header10_wParameterIdSub_WR(pv,w)   WR16(pv, w,24)
#define Header10_wParameterIdSub_RD(pv)     RD16(pv,   24)

#define Parameter1_0_szSerialNumber_RW(pv)   (char*)(pv)+26
#define Parameter1_0_dRangeUnit_WR(pv,d)     WRFP(pv, d,34)
#define Parameter1_0_dRangeUnit_RD(pv)       RDFP(pv,   34)
#define Parameter1_0_dAngleUnit_WR(pv,d)     WRFP(pv, d,38)
#define Parameter1_0_dAngleUnit_RD(pv)       RDFP(pv,   38)
#define Parameter1_0_bPolarAngleId_WR(pv,b)   WR8(pv, b,42)
#define Parameter1_0_bPolarAngleId_RD(pv)     RD8(pv,   42)
#define Parameter1_0_wLinesPerImage_WR(pv,w) WR16(pv, w,43)
#define Parameter1_0_wLinesPerImage_RD(pv)   RD16(pv,   43)
#define Parameter1_0_dEccenterDist_WR(pv,d)  WRFP(pv, d,45)
#define Parameter1_0_dEccenterDist_RD(pv)    RDFP(pv,   45)
#define Parameter1_0_dEccenterAngle_WR(pv,d) WRFP(pv, d,49)
#define Parameter1_0_dEccenterAngle_RD(pv)   RDFP(pv,   49)
#define Parameter1_1_dTimerUnit_WR(pv,d)     WRFP(pv, d,53)
#define Parameter1_1_dTimerUnit_RD(pv)       RDFP(pv,   53)
#define Parameter1_2_bTimeFormat_WR(pv,b)     WR8(pv, b,57)
#define Parameter1_2_bTimeFormat_RD(pv)       RD8(pv,   57)
#define Parameter1_3_dEccenterPolarAngle_WR(pv,d)   WRFP(pv, d, 58)
#define Parameter1_3_dEccenterPolarAngle_RD(pv)     RDFP(pv,    58)
#define Parameter1_3_bChangeWithPolarAngle_WR(pv,b)  WR8(pv, b, 62)
#define Parameter1_3_bChangeWithPolarAngle_RD(pv)    RD8(pv,    62)
#define Parameter1_4_dwEulerAnglePsi_WR(pv,b)  WR32(pv, b, 63)
#define Parameter1_4_dwEulerAnglePsi_RD(pv)    (DWORDINT)RD32(pv,    63)
#define Parameter1_4_dwEulerAngleTheta_WR(pv,b)  WR32(pv, b, 67)
#define Parameter1_4_dwEulerAngleTheta_RD(pv)    (DWORDINT)RD32(pv,    67)
#define Parameter1_4_dwEulerAnglePhi_WR(pv,b)  WR32(pv, b, 71)
#define Parameter1_4_dwEulerAnglePhi_RD(pv)    (DWORDINT)RD32(pv,    71)
#define Parameter1_4_dwTranslateX_WR(pv,b)  WR32(pv, b, 75)
#define Parameter1_4_dwTranslateX_RD(pv)    (DWORDINT)RD32(pv,    75)
#define Parameter1_4_dwTranslateY_WR(pv,b)  WR32(pv, b, 79)
#define Parameter1_4_dwTranslateY_RD(pv)    (DWORDINT)RD32(pv,    79)
#define Parameter1_4_dwTranslateZ_WR(pv,b)  WR32(pv, b, 83)
#define Parameter1_4_dwTranslateZ_RD(pv)    (DWORDINT)RD32(pv,    83)
#define Parameter1_5_wBeamAperture_WR(pv,w) WR16(pv, w, 87)
#define Parameter1_5_wBeamAperture_RD(pv)   RD16(pv,    87) 
#define Parameter1_5_wBeamDivergence_WR(pv,w) WR16(pv, w, 89)
#define Parameter1_5_wBeamDivergence_RD(pv)   RD16(pv,    89) 
#define Parameter1_5_wBeamFocus_WR(pv,w) WR16(pv, w, 91)
#define Parameter1_5_wBeamFocus_RD(pv)   RD16(pv,    91) 

#define Parameter2_0_szSerialNumber_RW(pv)   (char*)(pv)+26
#define Parameter2_0_dRangeUnit_WR(pv,d)     WRFP(pv, d,34)
#define Parameter2_0_dRangeUnit_RD(pv)       RDFP(pv,   34)
#define Parameter2_0_dTimerUnit_WR(pv,d)     WRFP(pv, d,38)
#define Parameter2_0_dTimerUnit_RD(pv)       RDFP(pv,   38)

#define Parameter3_0_szSerialNumber_RW(pv)   (char*)(pv)+26
#define Parameter3_0_dRangeUnit_WR(pv,d)     WRFP(pv, d,34)
#define Parameter3_0_dRangeUnit_RD(pv)       RDFP(pv,   34)
#define Parameter3_0_dAngleUnit_WR(pv,d)     WRFP(pv, d,38)
#define Parameter3_0_dAngleUnit_RD(pv)       RDFP(pv,   38)
#define Parameter3_0_dTimerUnit_WR(pv,d)     WRFP(pv, d,42)
#define Parameter3_0_dTimerUnit_RD(pv)       RDFP(pv,   42)
#define Parameter3_0_bAngleId_WR(pv,b)        WR8(pv, b,46)
#define Parameter3_0_bAngleId_RD(pv)          RD8(pv,   46)

/* Begin HE, 08.07.2002 */
#define Parameter4_0_szSerialNumber_RW(pv)   (char*)(pv)+26
#define Parameter4_0_dRangeUnit_WR(pv,d)     WRFP(pv, d,34)
#define Parameter4_0_dRangeUnit_RD(pv)       RDFP(pv,   34)
#define Parameter4_0_dAngleUnit_WR(pv,d)     WRFP(pv, d,38)
#define Parameter4_0_dAngleUnit_RD(pv)       RDFP(pv,   38)
#define Parameter4_0_dTimerUnit_WR(pv,d)     WRFP(pv, d,42)
#define Parameter4_0_dTimerUnit_RD(pv)       RDFP(pv,   42)
#define Parameter4_0_bMirrorType_WR(pv,b)    WRBITS(pv, b, 46, 8, 6, 2)
#define Parameter4_0_bMirrorType_RD(pv)      RDBITS(pv,    46, 8, 6, 2)
#define Parameter4_0_bAngleId_WR(pv,b)       WRBITS(pv, b, 46, 8, 0, 6)
#define Parameter4_0_bAngleId_RD(pv)         RDBITS(pv,    46, 8, 0, 6)
#define Parameter4_1_nHWRes_WR(pv,n)         WR8(pv, n, 47)
#define Parameter4_1_nHWRes_RD(pv)           RD8(pv,    47)
#define Parameter4_1_nTargetSelect_WR(pv,n)  WR8(pv, n, 48)
#define Parameter4_1_nTargetSelect_RD(pv)    RD8(pv,    48)
/* End HE, 08.07.2002 */

#define Parameter5_0_szSerialNumber_RW(pv)     (char*)(pv)+26
#define Parameter5_0_dRangeUnit_RD(pv)         RDFP(pv,   34)
#define Parameter5_0_dAngleUnit_RD(pv)         RDFP(pv,   38)
#define Parameter5_0_bNumberFacets_RD(pv)      RDBITS(pv, 42, 8, 0, 6)    
#define Parameter5_0_bMirrorType_RD(pv)        RDBITS(pv, 42, 8, 6, 2)
#define Parameter5_0_wLinesPerImage_RD(pv)     RD16(pv,	  43) 
#define Parameter5_0_dTimerUnit_RD(pv)         RDFP(pv,   45)
#define Parameter5_0_wBeamAperture_RD(pv)      RD16(pv,   49)
#define Parameter5_0_wBeamDivergence_RD(pv)    RD16(pv,   51)
#define Parameter5_0_wBeamFocus_RD(pv)         RD16(pv,   53)
#define Parameter5_0_bTriggerMode_RD(pv)       RDBITS(pv, 55, 8, 0, 1)
#define Parameter5_0_dCorrLaserOriginX_RD(pv)  RDFP(pv,   56)
#define Parameter5_0_dCorrLaserOriginY_RD(pv)  RDFP(pv,   60)
#define Parameter5_0_dCorrLaserOriginZ_RD(pv)  RDFP(pv,   64)
#define Parameter5_0_dCorrLaserDirX_RD(pv)     RDFP(pv,   68)
#define Parameter5_0_dCorrLaserDirY_RD(pv)     RDFP(pv,   72)
#define Parameter5_0_dCorrLaserDirZ_RD(pv)     RDFP(pv,   76)
#define Parameter5_0_dCorrMirrorAxisX_RD(pv)   RDFP(pv,   80)
#define Parameter5_0_dCorrMirrorAxisY_RD(pv)   RDFP(pv,   84)
#define Parameter5_0_dCorrMirrorAxisZ_RD(pv)   RDFP(pv,   88)
#define Parameter5_0_dCorrOffsetX_RD(pv)       RDFP(pv,   92)
#define Parameter5_0_dCorrOffsetY_RD(pv)       RDFP(pv,   96)
#define Parameter5_0_dCorrOffsetZ_RD(pv)       RDFP(pv,  100)
#define Parameter5_0_dCorrFacetNormal1X_RD(pv) RDFP(pv,  104)
#define Parameter5_0_dCorrFacetNormal1Y_RD(pv) RDFP(pv,  108)
#define Parameter5_0_dCorrFacetNormal1Z_RD(pv) RDFP(pv,  112)
#define Parameter5_0_dCorrFacetDist1_RD(pv)    RDFP(pv,  116)
#define Parameter5_0_dCorrFacetNormal2X_RD(pv) RDFP(pv,  120)
#define Parameter5_0_dCorrFacetNormal2Y_RD(pv) RDFP(pv,  124)
#define Parameter5_0_dCorrFacetNormal2Z_RD(pv) RDFP(pv,  128)
#define Parameter5_0_dCorrFacetDist2_RD(pv)    RDFP(pv,  132)
#define Parameter5_0_dCorrFacetNormal3X_RD(pv) RDFP(pv,  136)
#define Parameter5_0_dCorrFacetNormal3Y_RD(pv) RDFP(pv,  140)
#define Parameter5_0_dCorrFacetNormal3Z_RD(pv) RDFP(pv,  144)
#define Parameter5_0_dCorrFacetDist3_RD(pv)    RDFP(pv,  148)
#define Parameter5_0_dCorrFacetNormal4X_RD(pv) RDFP(pv,  152)
#define Parameter5_0_dCorrFacetNormal4Y_RD(pv) RDFP(pv,  156)
#define Parameter5_0_dCorrFacetNormal4Z_RD(pv) RDFP(pv,  160)
#define Parameter5_0_dCorrFacetDist4_RD(pv)    RDFP(pv,  164)

#define OFSM10(ph,pd,i) ((BYTE*)(Header10_wMeasOffset_RD(ph)+OFS(pd,i,Header10_wMeasSize_RD(ph))))
#define OFST10(ph,pd) ((BYTE*)(Header10_wMeasOffset_RD(ph)+OFS(pd,Header10_wMeasCount_RD(ph),Header10_wMeasSize_RD(ph))))

#define Meas1_0_wRange_WR(ph,pd,i,w)         WR16(OFSM10(ph,pd,i), w, 0)
#define Meas1_0_wRange_RD(ph,pd,i)           RD16(OFSM10(ph,pd,i),    0)
#define Meas1_0_wPolar_WR(ph,pd,i,w)         WR16(OFSM10(ph,pd,i), w, 2)
#define Meas1_0_wPolar_RD(ph,pd,i)           RD16(OFSM10(ph,pd,i),    2)
#define Meas1_0_wAzimuth_WR(ph,pd,i,w)       WR16(OFSM10(ph,pd,i), w, 4)
#define Meas1_0_wAzimuth_RD(ph,pd,i)         RD16(OFSM10(ph,pd,i),    4)
#define Meas1_0_fUpscan_WR(ph,pd,i,f)        WRBITS(OFSM10(ph,pd,i), f, 6, 16, 0, 1)
#define Meas1_0_fUpscan_RD(ph,pd,i)          RDBITS(OFSM10(ph,pd,i),    6, 16, 0, 1)
#define Meas1_0_fValid_WR(ph,pd,i,f)		 WRBITS(OFSM10(ph,pd,i), f, 6, 16, 1, 1)
#define Meas1_0_fValid_RD(ph,pd,i)			 RDBITS(OFSM10(ph,pd,i),    6, 16, 1, 1)
#define Meas1_0_wIntensity_WR(ph,pd,i,w)     WRBITS(OFSM10(ph,pd,i), w, 6, 16, 4,12)
#define Meas1_0_wIntensity_RD(ph,pd,i)       RDBITS(OFSM10(ph,pd,i),    6, 16, 4,12)

#define Meas1_1_fValidRGB_WR(ph,pd,i,f)    WRBITS(OFSM10(ph,pd,i), f, 8,  8, 7, 1)
#define Meas1_1_fValidRGB_RD(ph,pd,i)      RDBITS(OFSM10(ph,pd,i),    8,  8, 7, 1) 
#define Meas1_1_bShift_WR(ph,pd,i,b)       WRBITS(OFSM10(ph,pd,i), b, 8,  8, 0, 4)
#define Meas1_1_bShift_RD(ph,pd,i)         RDBITS(OFSM10(ph,pd,i),    8,  8, 0, 4)
#define Meas1_1_bRed_WR(ph,pd,i,b)            WR8(OFSM10(ph,pd,i), b, 9)
#define Meas1_1_bRed_RD(ph,pd,i)              RD8(OFSM10(ph,pd,i),    9)
#define Meas1_1_bGreen_WR(ph,pd,i,b)          WR8(OFSM10(ph,pd,i), b,10)
#define Meas1_1_bGreen_RD(ph,pd,i)            RD8(OFSM10(ph,pd,i),   10)
#define Meas1_1_bBlue_WR(ph,pd,i,b)           WR8(OFSM10(ph,pd,i), b,11)
#define Meas1_1_bBlue_RD(ph,pd,i)             RD8(OFSM10(ph,pd,i),   11)

#define Meas2_0_dwRange_WR(ph,pd,i,w)        WR24(OFSM10(ph,pd,i), w, 0)
#define Meas2_0_dwRange_RD(ph,pd,i)          RD24(OFSM10(ph,pd,i),    0)
#define Meas2_0_dwPolar_WR(ph,pd,i,w)        WR24(OFSM10(ph,pd,i), w, 3)
#define Meas2_0_dwPolar_RD(ph,pd,i)          RD24(OFSM10(ph,pd,i),    3)
#define Meas2_0_dwAzimuth_WR(ph,pd,i,w)      WR24(OFSM10(ph,pd,i), w, 6)
#define Meas2_0_dwAzimuth_RD(ph,pd,i)        RD24(OFSM10(ph,pd,i),    6)
#define Meas2_0_fUpscan_WR(ph,pd,i,f)        WRBITS(OFSM10(ph,pd,i), f, 9, 16, 0, 1)
#define Meas2_0_fUpscan_RD(ph,pd,i)          RDBITS(OFSM10(ph,pd,i),    9, 16, 0, 1)
#define Meas2_0_fValid_WR(ph,pd,i,f)		 WRBITS(OFSM10(ph,pd,i), f, 9, 16, 1, 1)
#define Meas2_0_fValid_RD(ph,pd,i)			 RDBITS(OFSM10(ph,pd,i),    9, 16, 1, 1)
#define Meas2_0_wIntensity_WR(ph,pd,i,w)     WRBITS(OFSM10(ph,pd,i), w, 9, 16, 4,12)
#define Meas2_0_wIntensity_RD(ph,pd,i)       RDBITS(OFSM10(ph,pd,i),    9, 16, 4,12)

#define Meas2_1_wRed_WR(ph,pd,i,w)            WR16(OFSM10(ph,pd,i), w, 11)
#define Meas2_1_wRed_RD(ph,pd,i)              RD16(OFSM10(ph,pd,i),    11)
#define Meas2_1_wGreen_WR(ph,pd,i,w)          WR16(OFSM10(ph,pd,i), w,13)
#define Meas2_1_wGreen_RD(ph,pd,i)            RD16(OFSM10(ph,pd,i),   13)
#define Meas2_1_wBlue_WR(ph,pd,i,w)           WR16(OFSM10(ph,pd,i), w,15)
#define Meas2_1_wBlue_RD(ph,pd,i)             RD16(OFSM10(ph,pd,i),   15)

#define Meas3_0_dwRange_WR(ph,pd,i,w)        WR24(OFSM10(ph,pd,i), w, 0)
#define Meas3_0_dwRange_RD(ph,pd,i)          RD24(OFSM10(ph,pd,i),    0)
#define Meas3_0_bIntensity_WR(ph,pd,i,b)      WR8(OFSM10(ph,pd,i), b, 3)
#define Meas3_0_bIntensity_RD(ph,pd,i)        RD8(OFSM10(ph,pd,i),    3)
#define Meas3_0_wAzimuth_WR(ph,pd,i,w)       WR16(OFSM10(ph,pd,i), w, 4)
#define Meas3_0_wAzimuth_RD(ph,pd,i)         RD16(OFSM10(ph,pd,i),    4)
#define Meas3_0_wPolar_WR(ph,pd,i,w)         WR16(OFSM10(ph,pd,i), w, 6)
#define Meas3_0_wPolar_RD(ph,pd,i)           RD16(OFSM10(ph,pd,i),    6)
#define Meas3_1_wStdDev_WR(ph,pd,i,w)		 WR16(OFSM10(ph,pd,i), w, 8)
#define Meas3_1_wStdDev_RD(ph,pd,i)			 RD16(OFSM10(ph,pd,i),    8)

#define Meas4_0_dwRange_WR(ph,pd,i,w)        WR24(OFSM10(ph,pd,i), w, 0)
#define Meas4_0_dwRange_RD(ph,pd,i)          RD24(OFSM10(ph,pd,i),    0)
#define Meas4_0_bIntensity_WR(ph,pd,i,b)      WR8(OFSM10(ph,pd,i), b, 3)
#define Meas4_0_bIntensity_RD(ph,pd,i)        RD8(OFSM10(ph,pd,i),    3)
#define Meas4_0_wAzimuth_WR(ph,pd,i,w)       WR16(OFSM10(ph,pd,i), w, 4)
#define Meas4_0_wAzimuth_RD(ph,pd,i)         RD16(OFSM10(ph,pd,i),    4)
#define Meas4_0_wPolar_WR(ph,pd,i,w)         WR16(OFSM10(ph,pd,i), w, 6)
#define Meas4_0_wPolar_RD(ph,pd,i)           RD16(OFSM10(ph,pd,i),    6)
#define Meas4_0_wRed_WR(ph,pd,i,w)            WR16(OFSM10(ph,pd,i), w, 8)
#define Meas4_0_wRed_RD(ph,pd,i)              RD16(OFSM10(ph,pd,i),    8)
#define Meas4_0_wGreen_WR(ph,pd,i,w)          WR16(OFSM10(ph,pd,i), w,10)
#define Meas4_0_wGreen_RD(ph,pd,i)            RD16(OFSM10(ph,pd,i),   10)
#define Meas4_0_wBlue_WR(ph,pd,i,w)           WR16(OFSM10(ph,pd,i), w,12)
#define Meas4_0_wBlue_RD(ph,pd,i)             RD16(OFSM10(ph,pd,i),   12)

/* variable sub id definitions */
#define DYNOFS(rd, ph, mask, def) (((Header10_wMeasIdSub_RD(ph))&(mask))? rd : def)

/* initialize the array of dynamic offsets */
#define Meas128_X_OFS(ph, ofs) { int n; n=0; \
ofs[0] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x01)?2:0; \
ofs[2] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x04)?1:0; \
ofs[3] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x08)?2:0; \
ofs[4] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x10)?2:0; \
ofs[5] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x20)?1:0; \
ofs[6] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x40)?3:0; \
}

#define Meas128_X_wRange_WR(ph,pd,i,ofs,w)            WR16(OFSM10(ph,pd,i), w, ofs[0])
#define Meas128_X_wRange_RD(ph,pd,i,ofs)       DYNOFS(RD16(OFSM10(ph,pd,i),    ofs[0]), ph, 0x01, 0)
#define Meas128_X_bAmplitude_WR(ph,pd,i,ofs,b)        WR8( OFSM10(ph,pd,i), b, ofs[2])
#define Meas128_X_bAmplitude_RD(ph,pd,i,ofs)   DYNOFS(RD8( OFSM10(ph,pd,i),    ofs[2]), ph, 0x04, 0)
#define Meas128_X_wPolar_WR(ph,pd,i,ofs,w)            WR16(OFSM10(ph,pd,i), w, ofs[3])
#define Meas128_X_wPolar_RD(ph,pd,i,ofs)       DYNOFS(RD16(OFSM10(ph,pd,i),    ofs[3]), ph, 0x08, 0)
#define Meas128_X_wAzimuth_WR(ph,pd,i,ofs,w)          WR16(OFSM10(ph,pd,i), w, ofs[4])
#define Meas128_X_wAzimuth_RD(ph,pd,i,ofs)     DYNOFS(RD16(OFSM10(ph,pd,i),    ofs[4]), ph, 0x10, 0)
#define Meas128_X_bQuality_WR(ph,pd,i,ofs,b)          WR8( OFSM10(ph,pd,i), b, ofs[5])
#define Meas128_X_bQuality_RD(ph,pd,i,ofs)     DYNOFS(RD8( OFSM10(ph,pd,i),    ofs[5]), ph, 0x20, 0)
#define Meas128_X_dwTimer_WR(ph,pd,i,ofs,w)           WR24(OFSM10(ph,pd,i), w, ofs[6])
#define Meas128_X_dwTimer_RD(ph,pd,i,ofs)      DYNOFS(RD24(OFSM10(ph,pd,i),    ofs[6]), ph, 0x40, 0)

/* Begin HE, 08.07.2002 */
/* initialize the array of dynamic offsets */
/* RS 12.08.2002 */
#define Meas129_X_OFS(ph, ofs) { int n; n=0; \
ofs[0] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x01)?3:0; \
ofs[2] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x04)?1:0; \
ofs[3] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x08)?3:0; \
ofs[5] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x20)?1:0; \
ofs[6] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x40)?3:0; \
ofs[7] = n; n += (Header10_wMeasIdSub_RD(ph) & 0x80)?6:0; \
}

#define Meas129_X_dwRange_WR(ph,pd,i,ofs,dw)          WR24(OFSM10(ph,pd,i), dw, ofs[0])
#define Meas129_X_dwRange_RD(ph,pd,i,ofs)      DYNOFS(RD24(OFSM10(ph,pd,i),     ofs[0]), ph, 0x01, 0)
#define Meas129_X_bAmplitude_WR(ph,pd,i,ofs,b)        WR8( OFSM10(ph,pd,i), b, ofs[2])
#define Meas129_X_bAmplitude_RD(ph,pd,i,ofs)   DYNOFS(RD8( OFSM10(ph,pd,i),    ofs[2]), ph, 0x04, 0)
#define Meas129_X_dwPolar_WR(ph,pd,i,ofs,dw)          WR24(OFSM10(ph,pd,i), w, ofs[3])
#define Meas129_X_dwPolar_RD(ph,pd,i,ofs)      DYNOFS(RD24(OFSM10(ph,pd,i),    ofs[3]), ph, 0x08, 0)
#define Meas129_X_bQuality_WR(ph,pd,i,ofs,b)          WR8( OFSM10(ph,pd,i), b, ofs[5])
#define Meas129_X_bQuality_RD(ph,pd,i,ofs)     DYNOFS(RD8( OFSM10(ph,pd,i),    ofs[5]), ph, 0x20, 0)
#define Meas129_X_dwTimer_WR(ph,pd,i,ofs,dw)          WR24(OFSM10(ph,pd,i), dw, ofs[6])
#define Meas129_X_dwTimer_RD(ph,pd,i,ofs)      DYNOFS(RD24(OFSM10(ph,pd,i),     ofs[6]), ph, 0x40, 0)
#define Meas129_X_wRed_WR(ph,pd,i,ofs,w)              WR16(OFSM10(ph,pd,i), dw, ofs[7])
#define Meas129_X_wRed_RD(ph,pd,i,ofs)         DYNOFS(RD16(OFSM10(ph,pd,i),     ofs[7]), ph, 0x80, 0)
#define Meas129_X_wGreen_WR(ph,pd,i,ofs,w)            WR16(OFSM10(ph,pd,i), dw, ofs[7]+2)
#define Meas129_X_wGreen_RD(ph,pd,i,ofs)       DYNOFS(RD16(OFSM10(ph,pd,i),     ofs[7]+2), ph, 0x80, 0)
#define Meas129_X_wBlue_WR(ph,pd,i,ofs,w)             WR16(OFSM10(ph,pd,i), dw, ofs[7]+4)
#define Meas129_X_wBlue_RD(ph,pd,i,ofs)        DYNOFS(RD16(OFSM10(ph,pd,i),     ofs[7]+4), ph, 0x80, 0)
/* End HE, 08.07.2002 */

#define Trailer1_0_wLineNumber_WR(ph,pd,w)   WR16(OFST10(ph,pd), w, 0)
#define Trailer1_0_wLineNumber_RD(ph,pd)     RD16(OFST10(ph,pd),    0)
#define Trailer1_0_fUpscan_WR(ph,pd,f)       WRBITS(OFST10(ph,pd), f, 6, 16, 0, 1)
#define Trailer1_0_fUpscan_RD(ph,pd)         RDBITS(OFST10(ph,pd),    6, 16, 0, 1)
#define Trailer1_0_fFrame_WR(ph,pd,f)        WRBITS(OFST10(ph,pd), f, 6, 16, 3, 1)
#define Trailer1_0_fFrame_RD(ph,pd)          RDBITS(OFST10(ph,pd),    6, 16, 3, 1)

#define Trailer2_0_wLineNumber_WR(ph,pd,w)   WR16(OFST10(ph,pd), w, 0)
#define Trailer2_0_wLineNumber_RD(ph,pd)     RD16(OFST10(ph,pd),    0)
#define Trailer2_0_dwSensorTimeStamp_WR(ph,pd,dw) WR32(OFST10(ph,pd), dw, 2)
#define Trailer2_0_dwSensorTimeStamp_RD(ph,pd)    RD32(OFST10(ph,pd),     2)
#define Trailer2_0_fUpscan_WR(ph,pd,f)       WRBITS(OFST10(ph,pd), f, 6, 16, 0, 1)
#define Trailer2_0_fUpscan_RD(ph,pd)         RDBITS(OFST10(ph,pd),    6, 16, 0, 1)
#define Trailer2_0_fFrame_WR(ph,pd,f)        WRBITS(OFST10(ph,pd), f, 6, 16, 3, 1)
#define Trailer2_0_fFrame_RD(ph,pd)          RDBITS(OFST10(ph,pd),    6, 16, 3, 1)

#define Trailer3_0_wLineNumber_WR(ph,pd,dw)  WR16(OFST10(ph,pd), dw, 0)
#define Trailer3_0_wLineNumber_RD(ph,pd)     RD16(OFST10(ph,pd),     0)
#define Trailer3_0_fAzimuthUp_WR(ph,pd,f)    WRBITS(OFST10(ph,pd), f,3, 8, 0, 1)
#define Trailer3_0_fAzimuthUp_RD(ph,pd)      RDBITS(OFST10(ph,pd),   3, 8, 0, 1)
#define Trailer3_0_fPolarUp_WR(ph,pd,f)      WRBITS(OFST10(ph,pd), f,3, 8, 4, 1)
#define Trailer3_0_fPolarUp_RD(ph,pd)        RDBITS(OFST10(ph,pd),   3, 8, 4, 1)
#define Trailer3_0_fFrame_WR(ph,pd,f)        WRBITS(OFST10(ph,pd), f,3, 8, 3, 1)
#define Trailer3_0_fFrame_RD(ph,pd)          RDBITS(OFST10(ph,pd),   3, 8, 3, 1)

#define Trailer4_0_dwTimer_WR(ph,pd,dw)      WR24(OFST10(ph,pd), dw, 0)
#define Trailer4_0_dwTimer_RD(ph,pd)         RD24(OFST10(ph,pd),     0)

#define Trailer5_0_wLineNumber_WR(ph,pd,w)   WR16(OFST10(ph,pd), w, 0)
#define Trailer5_0_wLineNumber_RD(ph,pd)     RD16(OFST10(ph,pd),    0)
#define Trailer5_0_dwSensorTimeStamp_WR(ph,pd,dw) WR32(OFST10(ph,pd), dw, 2)
#define Trailer5_0_dwSensorTimeStamp_RD(ph,pd)    RD32(OFST10(ph,pd),     2)
#define Trailer5_0_fUpscan_WR(ph,pd,f)       WRBITS(OFST10(ph,pd), f, 6, 16, 0, 1)
#define Trailer5_0_fUpscan_RD(ph,pd)         RDBITS(OFST10(ph,pd),    6, 16, 0, 1)
#define Trailer5_0_fFrame_WR(ph,pd,f)        WRBITS(OFST10(ph,pd), f, 6, 16, 3, 1)
#define Trailer5_0_fFrame_RD(ph,pd)          RDBITS(OFST10(ph,pd),    6, 16, 3, 1)
#define Trailer5_0_fLineDown_WR(ph,pd,f)     WRBITS(OFST10(ph,pd), f, 6, 16, 4, 1)
#define Trailer5_0_fLineDown_RD(ph,pd)       RDBITS(OFST10(ph,pd),    6, 16, 4, 1)

#define Trailer6_0_bScanStatus_RD(ph,pd)     RD8(OFST10(ph,pd),     0)
#define Trailer6_0_wLineCount_RD(ph,pd)      RD16(OFST10(ph,pd),    1)
#define Trailer6_1_dwSyncCounter_RD(ph,pd)   RD24(OFST10(ph,pd),    3)
#define Trailer6_1_dwSyncTimer_RD(ph,pd)     RD24(OFST10(ph,pd),    6)
#define Trailer6_2_bTemperature_RD(ph,pd)    RD8(OFST10(ph,pd),     9)
#define Trailer6_3_dwTiltAngleY_RD(ph,pd)       RD24(OFST10(ph,pd),   10)
#define Trailer6_3_dwTiltAngleZ_RD(ph,pd)       RD24(OFST10(ph,pd),   13)


#ifdef __cplusplus
}
#endif

#endif /* __SCANDSYS_H__ */
